package net.sf.gluebooster.demos.pojo.clock;

import java.awt.BorderLayout;
import java.io.Closeable;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JFrame;

import net.sf.gluebooster.java.booster.basic.gui.swing.SwingFactory;

/**
 * {@Feature}Display of a digital clock.{@FeatureEnd}
 * 
 * 
 * @author CBauer
 *
 */
public class DigitalClock implements Runnable, Closeable {

	/**
	 * The frame that contains the digital clock
	 */
	private JFrame frame;

	/**
	 * Should the clock stop
	 */
	private boolean stop = false;

	/**
	 * Formatter of the displayed time
	 */
	private static SimpleDateFormat formatter = new SimpleDateFormat("HH:mm:ss");

	private DigitalClock() {

	}

	private DigitalClock(JFrame frame) {
		this.frame = frame;
	}

	/**
	 * Starts the clock.
	 * 
	 * @param ignored
	 *            arguments are ignored.
	 */
	public static void main(String[] ignored) {

		displayDigitalClockInForeground();

	}

	/**
	 * Displays a small digital clock always in the foreground.
	 * 
	 * @return the frame displaying the clock
	 */
	public static JFrame displayDigitalClockInForeground() {
		JFrame frame = new SwingFactory().createFrame("Clock", 200, 0, true,
				new BorderLayout());
		frame.setVisible(true);
		frame.setAlwaysOnTop(true);

		new Thread(new DigitalClock(frame)).start();

		return frame;
	}

	@Override
	public void run() {
		while (frame.isVisible() && !stop) {
			try {
				Thread.sleep(200);
			} catch (Exception ex) {

			}

			frame.setTitle(formatter.format(new Date()));

		}

		frame.dispose();

	}

	@Override
	public void close() {
		stop = true;

	}

}
