package net.sf.gluebooster.demos.pojo.flashcards;

import java.awt.Font;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JLabel;

import net.sf.gluebooster.demos.pojo.languages.VocabularyEntry;
import net.sf.gluebooster.demos.pojo.refactor.AppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.AppDisplayBySwingBorderLayout;
import net.sf.gluebooster.demos.pojo.refactor.AppViewAbstraction;
import net.sf.gluebooster.java.booster.essentials.utils.ContainerBoostUtils;

/**
 * {@Feature}An demo application to learn with flashcards.{@FeatureEnd}
 * 
 * @author cbauer
 *
 */
public class FlashcardDemo {

	/**
	 * create a demo instance with some chinese flashcards.
	 * 
	 * @return demo model
	 */
	public static Map<Object, Object> createChineseDemoModel() throws Exception {
		List/* <VocabularyEntry> */ cards = Arrays.asList(new VocabularyEntry("〇", "ling2", "zero"), new VocabularyEntry("一", "yi1", "one"),
				new VocabularyEntry("下", "xia", "below"));
		Flashcards<Object> flashcards = new Flashcards<Object>();
		flashcards.setCardsAtLevel0(cards);
		flashcards.setNames(ContainerBoostUtils.createMap(0, "Chinese", 1, "Pinyin", 2, "English"));
		flashcards.setFrontSideIndices(new int[] { 0 });
		flashcards.setBackSideIndices(new int[] { 1, 2 });
	
		HashMap<Object, Object> model = new HashMap();
		model.put(AppDefinition.FIELD_TITLE, "Chinese Vocabulary");
		model.put(FlashcardAppDefinition.FIELD_ALL_FLASHCARDS, flashcards);
	
		return model;
	
	}

	/**
	 * Starts the demo
	 * 
	 * @param ignored
	 *            the parameters are ignored
	 */
	public static void main(String[] ignored) throws Exception {
		startSimpleSwingDemo(null);
	}

	/**
	 * Creates a swing layout of the application
	 * 
	 * @param view
	 *            the view of the application
	 * @return the created layout
	 */
	private static AppDisplayBySwingBorderLayout createSwingLayout(FlashcardsAppView view) throws Exception {
		AppDisplayBySwingBorderLayout result = new AppDisplayBySwingBorderLayout(view);

		JLabel label = new JLabel();
		label.setFont(new Font(label.getFont().getName(), Font.PLAIN, 36));
		label.addMouseListener(view);
		result.putComponentIntoDisplayModel(FlashcardAppDefinition.FIELD_DISPLAYED_FLASHCARD_SIDE, label);
		// getDisplayModel().put(FlashcardAppDefinition.FIELD_DISPLAYED_FLASHCARD_SIDE, new ComponentHolder(label));
		result.setCentral(label);

		result.setNorth(result.putComponentIntoDisplayModel(FlashcardAppDefinition.FIELD_FLASHCARDS_STATISTICS,
				new JLabel("no cards loaded yet, only demo cards available")));

		result.addToMenuBar(
				result.createMenu("Flashcards", "Save", FlashcardAppDefinition.COMMAND_SAVE_CARDS, "Load", FlashcardAppDefinition.COMMAND_LOAD_CARDS));
		return result;
	}

	/**
	 * Creates a swing demo with some chinese flashcards if no model is provided
	 * 
	 * @param optionalModel
	 *            an initial model (optional) of the application.
	 * @return the view of the demo.
	 */
	public static AppDisplayBySwingBorderLayout startSimpleSwingDemo(Map optionalModel) throws Exception {
		if (optionalModel == null) {
			optionalModel = createChineseDemoModel();
		}

		FlashcardsAppController<Object> controller = new FlashcardsAppController<Object>(optionalModel);
		FlashcardsAppView view = new FlashcardsAppView();
		AppDisplayBySwingBorderLayout display = createSwingLayout(view);
		// FlashcardsAppDisplaySwing display = new FlashcardsAppDisplaySwing(view);
		AppViewAbstraction.startApplication(view, controller);
		return display;

		// Map appModel = createChineseDemoModel();
		// return AbstractAppView.startApplication(FlashcardsAppController.class, appModel, FlashcardsAppView.class, FlashcardsAppDisplaySwing.class);
	}

}
