package net.sf.gluebooster.demos.pojo.flashcards;

import java.awt.event.MouseEvent;
import java.io.File;
import java.util.Map;

import javax.swing.JOptionPane;

import net.sf.gluebooster.demos.pojo.refactor.AppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.CopyBoostUtils;
import net.sf.gluebooster.demos.pojo.refactor.gb.AppViewImpl;
import net.sf.gluebooster.java.booster.basic.events.UserInteraction;
import net.sf.gluebooster.java.booster.essentials.utils.SwingBoostUtils;
import net.sf.gluebooster.java.booster.essentials.utils.ThrowableBoostUtils;

/**
 * View logic without gui.
 * 
 * @author cbauer
 *
 */
public class FlashcardsAppView<CardElement> extends AppViewImpl<FlashcardsAppController<CardElement>, FlashcardAppDefinition> {

	/**
	 * The state that a frontside word of the flashcard is displayed. TODO make private
	 */
	static final int STATE_DISPLAYING_WORD = 1;
	/**
	 * The state that the backside translation of the word of the flashcard is displayed.
	 * 
	 * TODO make private
	 */
	static final int STATE_DISPLAYING_TRANSLATION = 2;


	public FlashcardsAppView() throws Exception {
		super(FlashcardAppDefinition.singleton.getValue());
	}


	/**
	 * Lets the controller execute a command.
	 * 
	 * @param command
	 *            the command to execute
	 * @param parameter
	 *            parameter of the command
	 * @return the result of the command
	 */
	private Map<Object, Object> controllerExecute(Object command, Object parameter) throws Exception {
		return getController().executeCommand(command, parameter);

	}

	@Override
	public boolean execute(Object commandName, Object parameter) throws Exception {
		// TODO display feedback
		if (FlashcardAppDefinition.COMMAND_SAVE_CARDS.equals(commandName)) {
			File result = SwingBoostUtils.chooseFile("Save configuration in", false);
			if (result != null) {
				if (result.exists()) {
					if (JOptionPane.YES_OPTION == ((UserInteraction) getDisplayModel().get(AppDefinition.USER_INTERACTION))
							.showConfirmDialog(null, "File exists. Overwrite?", "Overwrite cards",
							JOptionPane.OK_CANCEL_OPTION)) {
						result.delete();
					} else {
						// return false;

					}
				}
				getController().executeCommand(FlashcardAppDefinition.COMMAND_SAVE_CARDS, result);
			}
			// return result;
		} else if (FlashcardAppDefinition.COMMAND_LOAD_CARDS.equals(commandName)) {
			File result = SwingBoostUtils.chooseFile("Load configuration from", false);
			if (result != null && result.exists()) {
				getController().executeCommand(FlashcardAppDefinition.COMMAND_LOAD_CARDS, result);
				// TODO display feedback
				displayNextCard(null);
			}
			// return result;
		} else if (AppDefinition.COMMAND_START.equals(commandName)) {
			controllerExecute(AppDefinition.COMMAND_START, null);
		} else {
			return false;
			// throw new IllegalStateException("Unsupported command: " + commandName);
		}
		return true;

	}

	/**
	 * Lets the controller choose the next card and puts the result into the display model.
	 * 
	 * @param currentCardKnown
	 *            has the current card been known
	 */
	private void displayNextCard(Boolean currentCardKnown) throws Exception {

		Map<Object, Object> model = controllerExecute(FlashcardAppDefinition.COMMAND_NEXT_CARD, currentCardKnown);
		CopyBoostUtils.copyEntries(model, getDisplayModel(), FlashcardAppDefinition.FIELD_DISPLAYED_FLASHCARD_SIDE,
				FlashcardAppDefinition.FIELD_FLASHCARDS_STATISTICS);
		state = STATE_DISPLAYING_WORD;
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		try {
			Map model;
			Boolean currentCardKnown = null;
			switch (state) {
			case STATE_DISPLAYING_TRANSLATION:
				currentCardKnown = (event.getButton() == MouseEvent.BUTTON1);
				// true = known = button 1 ; false = not known = other button
				// continue
			case STATE_INITIAL:
				displayNextCard(currentCardKnown);
				break;
			case STATE_DISPLAYING_WORD:
				model = controllerExecute(AppDefinition.COMMAND_GET_MODEL, null);
				getDisplayModel().put(FlashcardAppDefinition.FIELD_DISPLAYED_FLASHCARD_SIDE,
						((Flashcards<CardElement>) model.get(FlashcardAppDefinition.FIELD_ALL_FLASHCARDS)).getCurrentBacksideString());
				state = STATE_DISPLAYING_TRANSLATION;
				break;
			default:
				throw new IllegalStateException("event not supported in state " + state);
			}

		} catch (Exception ex) {
			throw ThrowableBoostUtils.toRuntimeException(ex);
		}
	}


}
