package net.sf.gluebooster.demos.pojo.languages.chinese;

import java.io.File;

import javax.swing.tree.DefaultMutableTreeNode;

import net.sf.gluebooster.demos.pojo.flashcards.FlashcardAppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.AppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.ObjectDescriptionBoostUtils;
import net.sf.gluebooster.demos.pojo.refactor.TransformerByInvoking;
import net.sf.gluebooster.demos.pojo.refactor.TransformerToConstant;
import net.sf.gluebooster.demos.pojo.refactor.ObjectDescription;

/**
 * Definition of an application to learn chinese.
 * 
 * TODO: Refactor the commands and fields, so that they contain help text what the constant means.
 * 
 * @author cbauer
 *
 */
public class ChineseAppDefinition extends AppDefinition {

	/**
	 * The singleton instance. No instanciation yet, because the other static constants are not yet defined;
	 */
	public static final TransformerToConstant<ChineseAppDefinition> singleton = TransformerToConstant
			.createWithInitializer(TransformerByInvoking.createNewInstanceInvokingTransformer(ChineseAppDefinition.class));

	/**
	 * The command to convert traditional chinese characters into their simplified form.
	 */
	public static final String COMMAND_CHANGE_TRADITIONAL_TO_SIMPLIFIED = "Traditional -> Simplified";

	/**
	 * The command to create the vocabulary from a given text.
	 */
	public static final String COMMAND_CREATE_VOCABULARY = "Create vocabulary";

	/**
	 * The command to extract the chinese characters from a given text.
	 */
	public static final String COMMAND_EXTRACT_CHARACTERS = "Extract characters";

	/**
	 * The command to sort the lines of a given text into another. Each line at the position where all characters are known
	 */
	public static final String COMMAND_SORT_TEXT = "Sort text";

	/**
	 * The name of the field of the original text that will be transformed
	 */
	public static final String FIELD_ORIGINAL_TEXT = "Original Text";

	/**
	 * The name of the field with the transformed text
	 */
	public static final String FIELD_TRANSFORMED_TEXT = "Transformed Text";

	/**
	 * Description of the corresponding configuration field.
	 */
	// public static final ObjectDescription CONFIGURATION_FIELD_CHINESE_FRENCH_FILE = ObjectDescriptionBoostUtils.create("Chinese French Dictionary File",
	// "The file that contains the chinese french dictionary", File.class);
	/**
	 * Description of the corresponding configuration field.
	 */
	public static final ObjectDescription CONFIGURATION_FIELD_CHINESE_LANGUAGE1_FILE = ObjectDescriptionBoostUtils.create("Chinese Language 1 Dictionary File",
			"The file that contains the chinese dictionary into language 1", File.class);
	/**
	 * Description of the corresponding configuration field.
	 */
	public static final ObjectDescription CONFIGURATION_FIELD_CHARACTER_COMPONENTS_FILE = ObjectDescriptionBoostUtils.create("Character Components File",
			"The file that contains the character components composition", File.class);
	/**
	 * Description of the corresponding configuration field.
	 */
	// public static final ObjectDescription CONFIGURATION_FIELD_CHINESE_TEXT_FILE = ObjectDescriptionBoostUtils.create("Chinese Text File",
	// "The file that contains the chinese text", File.class);
	/**
	 * Description of the corresponding configuration field.
	 */
	public static final ObjectDescription CONFIGURATION_FIELD_VOCABULARY_FOR_EACH_LINE = ObjectDescriptionBoostUtils.create("Vocabulary for each line",
			"Should the vocabulary created for each line (instead for the whole text)", Boolean.class);

	/**
	 * Description of the corresponding configuration field.
	 */
	public static final ObjectDescription CONFIGURATION_FIELD_DEFAULT_RESULT_FILE = ObjectDescriptionBoostUtils.create("Default file for results",
			"The content of the right side will be written into this file", File.class);

	public ChineseAppDefinition() {
		setTitle("Chinese");

		DefaultMutableTreeNode configurationRoot = new DefaultMutableTreeNode();
		// configurationRoot.add(new DefaultMutableTreeNode(CONFIGURATION_FIELD_CHINESE_FRENCH_FILE));
		configurationRoot.add(new DefaultMutableTreeNode(CONFIGURATION_FIELD_CHINESE_LANGUAGE1_FILE));
		configurationRoot.add(new DefaultMutableTreeNode(CONFIGURATION_FIELD_CHARACTER_COMPONENTS_FILE));
		// configurationRoot.add(new DefaultMutableTreeNode(CONFIGURATION_FIELD_CHINESE_TEXT_FILE));
		configurationRoot.add(new DefaultMutableTreeNode(CONFIGURATION_FIELD_VOCABULARY_FOR_EACH_LINE));
		configurationRoot.add(new DefaultMutableTreeNode(CONFIGURATION_FIELD_DEFAULT_RESULT_FILE));
		setConfiguration(configurationRoot);
	}


}
