package net.sf.gluebooster.demos.pojo.refactor;

import java.util.HashMap;
import java.util.Map;

import net.sf.gluebooster.java.booster.essentials.objects.BoostedObject;

/**
 * An abstract controller class for applications.
 * 
 * The controller holds a model that is returned on each command.
 * 
 * 
 * @author cbauer
 *
 * @param <CommandDescription>
 * @param <Model>
 */
public abstract class AppControllerAbstraction<CommandDescription> extends BoostedObject
		implements CommandHandler<CommandDescription, Object, Map<Object, Object>> {


	/**
	 * The model with the data.
	 */
	private Map<Object, Object> model = new HashMap<>();

	protected AppControllerAbstraction() {
	}

	public Map<Object, Object> getModel() {
		return model;
	}

	public void setModel(Map<Object, Object> model) {
		this.model = model;
	}


	/**
	 * @return Always returns the model.
	 */
	@Override
	public final Map<Object, Object> executeCommand(CommandDescription commandName, Object parameter) throws Exception {
		getLog().debug(getClass().getSimpleName(), ".executeCommand ", commandName, " ", parameter);
		// TODO check whether the command is supported or not

		if (!AppDefinition.COMMAND_GET_MODEL.equals(commandName)) {
			if (!executeCommandImpl(commandName, parameter)) {
				throw new IllegalStateException("command " + commandName + " not supported");
			}

		}
		return model;
	}

	/**
	 * Implementation of the command execution.
	 * 
	 * @param commandName
	 *            the name of the command.
	 * @param parameter
	 *            the parameter of the command
	 * @return true if the command has been fully handled
	 */
	protected abstract boolean executeCommandImpl(CommandDescription commandName, Object parameter) throws Exception;

}
