package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.io.File;
import java.util.Enumeration;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.tree.DefaultMutableTreeNode;

import org.apache.commons.lang3.tuple.Pair;

import edu.uci.ics.jung.visualization.transform.shape.Graphics2DWrapper;
import net.sf.gluebooster.demos.pojo.languages.chinese.ChineseAppDefinition;
import net.sf.gluebooster.java.booster.basic.gui.swing.SwingFactory;
import net.sf.gluebooster.java.booster.essentials.meta.ObjectDescription;

/**
 * Implementation of a gui-view using swing. The basic layout is a border layout. Subclasses must create the central panel and can optionally create the
 * north-,south-, east- and west panel.
 * 
 * @defaultParamText appDefinition the definition of the app
 * 
 * @author CBauer
 *
 * @param <Controller>
 */
public class AppDisplayBySwingBorderLayout extends BoostedObject implements CommandHandler {

	/**
	 * The view-logic. Defines the reactions to user input.
	 */
	private AppViewAbstraction viewLogic;

	/**
	 * The model used to display in the view. (Not necessarily the model used by the controller)
	 */
	private ObjectAttributes displayModel = new ObjectAttributes<>();

	/**
	 * A factory to create gui elements.
	 */
	private SwingFactory guiFactory = new SwingFactory();

	/**
	 * The frame of this display.
	 */
	private JFrame frame;

	/**
	 * The central component of the border layout.
	 */
	private Component central;

	/**
	 * The north component of the border layout.
	 */
	private Component north;

	/**
	 * The south component of the border layout.
	 */
	private Component south;

	/**
	 * The east component of the border layout.
	 */
	private Component east;

	/**
	 * The west component of the border layout.
	 */
	private Component west;

	/**
	 * The displayed menu bar.
	 */
	private JMenuBar menuBar = new JMenuBar();

	public JMenuBar getMenuBar() {
		return menuBar;
	}

	public void setMenuBar(JMenuBar menuBar) {
		this.menuBar = menuBar;
	}

	/**
	 * Adds a menu to the menu bar
	 * 
	 * @param menu
	 *            will be added
	 */
	public void addToMenuBar(JMenu menu) {
		getMenuBar().add(menu);
	}

	public Component getCentral() {
		return central;
	}

	public void setCentral(Component central) {
		this.central = central;
	}

	public Component getNorth() {
		return north;
	}

	public void setNorth(Component north) {
		this.north = north;
	}

	public Component getSouth() {
		return south;
	}

	public void setSouth(Component south) {
		this.south = south;
	}

	public Component getEast() {
		return east;
	}

	public void setEast(Component east) {
		this.east = east;
	}

	public Component getWest() {
		return west;
	}

	public void setWest(Component west) {
		this.west = west;
	}

	/**
	 * Constructor with values.
	 * 
	 * @param viewLogic
	 *            should be set here, because it may be used as listener during setup.
	 */
	public AppDisplayBySwingBorderLayout(AppViewAbstraction viewLogic) throws Exception {
		this.viewLogic = viewLogic;
		viewLogic.setDisplayModel(displayModel);
		// prefer an indirect access, because the components will be put into the fields
		displayModel.setPreferIndirect(true);
		displayModel.put(AppDefinition.USER_INTERACTION, new SwingUserInteraction());

		displayModel.put(AppDefinition.COMMAND_HANDLER, this);

	}

	public AppDisplayBySwingBorderLayout(AppViewAbstraction viewLogic, AppDefinition appDefinition) throws Exception {
		this(viewLogic);
		createConfigurationDialog(viewLogic.getAppDefinition());
	}

	/**
	 * Creates the dialog with the fields used for configuring the app.
	 */
	private void createConfigurationDialog(AppDefinition appDefinition) throws Exception {
		// handle configuration
		DefaultMutableTreeNode configuration = appDefinition.getConfiguration();
		if (configuration != null) {

			JDialog configurationDialog = new JDialog((Frame) null, "Configuration");
			configurationDialog.setModal(true);
			putVisibilityComponentIntoDisplayModel(AppDefinition.DIALOG_DISPLAY_CONFIGURATION, configurationDialog);

			Enumeration<DefaultMutableTreeNode> nodes = configuration.breadthFirstEnumeration();
			int fieldCounter = 0;

			Container configurationPane = configurationDialog.getContentPane();
			// configurationPane.setLayout(new GridLayout(fieldCounter, 2));
			configurationPane.setLayout(new GridBagLayout());

			GridBagConstraints left = new GridBagConstraints();
			left.weightx = 1;
			left.anchor = GridBagConstraints.WEST;

			GridBagConstraints right = new GridBagConstraints();
			right.weightx = 10;
			left.anchor = GridBagConstraints.WEST;
			right.gridwidth = GridBagConstraints.REMAINDER;
			right.fill = GridBagConstraints.HORIZONTAL;

			while (nodes.hasMoreElements()) {
				DefaultMutableTreeNode node = nodes.nextElement();
				ObjectDescription field = (ObjectDescription) node.getUserObject();
				// field may be null at the root, then it is ignored
				if (field != null) {
					fieldCounter++;
					Class valueClass = field.getValueDescription().getValueClass();

					Component labelComponent = new JLabel(field.getName());
					// configurationPane.add(labelComponent);
					configurationPane.add(labelComponent, left);
					
					Component valueComponent = null;

					if (File.class.equals(valueClass)) {
						
						Pair<ComponentHolder, Component> textfield = SwingBoostUtils.createFileTextfieldWithFileChooser();
								
//						JButton button = new JButton(field.getName());
//						JTextField textfield = new JTextField();
//						// connection from button to field value
//						TransformationGraph fieldValueSetter = TransformationGraph
//								.createChain(new TransformerToConstant<ConstantClass>(new ComponentHolder<>(textfield)));
//						button.setActionCommand(BasicCommands.COMMAND_START);
//						EventMulticaster listener = new EventMulticaster();
//						listener.addCommandHandler(fieldValueSetter);
//						button.addActionListener(listener);

//						labelComponent = button;
//						valueComponent = textfield;
						valueComponent = textfield.getRight();
						putComponentIntoDisplayModel(field, textfield.getLeft());

					} else {
						// default

						if (Boolean.class.equals(valueClass)) {
							valueComponent = new JCheckBox();
						} else {
							// default
							valueComponent = new JTextField();
						}
						putComponentIntoDisplayModel(field, valueComponent);
					}

					
					// configurationPane.add(valueComponent);
					configurationPane.add(valueComponent, right);
					
				}

			}

			DefaultActions dialogHide = DefaultActions.createSetInvisibleAction(configurationDialog);
			JButton cancelButton = new JButton("cancel");
			cancelButton.addActionListener(dialogHide);
			configurationDialog.getContentPane().add(cancelButton);
			JButton okButton = new JButton("ok");
			configurationDialog.getContentPane().add(okButton);
			okButton.setActionCommand(AppDefinition.COMMAND_EDITED_CONFIGURATION);
			EventMulticaster commandStarter = new EventMulticaster();
			commandStarter.addCommandHandler(viewLogic);
			commandStarter.addActionListener(dialogHide);
			okButton.addActionListener(commandStarter);

			fieldCounter++;
			// configurationPane.setLayout(new GridLayout(fieldCounter, 2));
			configurationDialog.setSize(1000, fieldCounter * 50);


			// currently only one level below the root is supported


		}

	}

	// @Override
	// public UserInteraction getUserInteraction() {
	// return userInteraction;
	// }
	//
	// public void setUserInteraction(UserInteraction userInteraction) {
	// this.userInteraction = userInteraction;
	// }

	/**
	 * Must be overwritten or the menu bar must be created using the getter/setter.
	 * 
	 * @param menuBar
	 */
	protected void fillMenuBar(JMenuBar menuBar) {

	}


	public SwingFactory getGuiFactory() {
		return guiFactory;
	}

	public void setGuiFactory(SwingFactory guiFactory) {
		this.guiFactory = guiFactory;
	}

	/**
	 * Adds a panel to the frame
	 * 
	 * @param frame
	 *            the frame the panel is added to
	 * @param panel
	 *            if null, then nothing is done
	 * @param constraint
	 *            constraint for adding
	 */
	private void addPanel(JFrame frame, Component panel, Object constraint) {
		if (panel != null) {
			frame.getContentPane().add(new JScrollPane(panel), constraint);
		}
	}

	/**
	 * Creates the panel for the layout. If any panel is not visible, try to put some visible parts into it from the beginning.
	 */
	private void init() throws Exception {
		String title = (String) displayModel.get(AppDefinition.FIELD_TITLE);
		int width = 800;
		int height = 500;
		frame = guiFactory.createFrame(title, width, height, true, new BorderLayout());


		fillMenuBar(menuBar);
		frame.setJMenuBar(menuBar);

		addPanel(frame, central, BorderLayout.CENTER);
		addPanel(frame, north, BorderLayout.NORTH);
		addPanel(frame, south, BorderLayout.SOUTH);
		addPanel(frame, east, BorderLayout.EAST);
		addPanel(frame, west, BorderLayout.WEST);


		frame.setVisible(true);

	}

	public AppViewAbstraction getViewLogic() {
		return viewLogic;
	}

	public void setViewLogic(AppViewAbstraction viewLogic) {
		this.viewLogic = viewLogic;
	}

	/**
	 * Creates a menu with multiple entries. The view logic is added as listener
	 * 
	 * @param title
	 *            the title of the menu
	 * @param labelCommands
	 *            [label1, command1, label2, command2, ...]
	 * @return the created menu
	 */
	public JMenu createMenu(String title, String... labelCommands) {
		JMenu resultMenu = new JMenu(title);

		for (int i = 0; i < labelCommands.length; i = i + 2) {
			guiFactory.addMenuItem(resultMenu, labelCommands[i], labelCommands[i + 1], viewLogic);
		}

		return resultMenu;
	}

	@Override
	public Object executeCommand(Object commandName, Object parameter) throws Exception {
		if (AppDefinition.COMMAND_START.equals(commandName)) {
			init();
			return null;
		}
		throw new IllegalStateException("command not supported: " + commandName);
	}

	public ObjectAttributes getDisplayModel() {
		return displayModel;
	}

	/**
	 * Create a button.
	 * 
	 * @param label
	 *            the label of the button
	 * @param command
	 *            the command to execute when the button is pressed
	 * @return the created button
	 */
	public Component createButton(String label, String command) {
		JButton result = new JButton(label);
		result.setActionCommand(command);
		result.addActionListener(viewLogic);
		return result;
	}

	/**
	 * Wraps a component into a holder (if necessary) and puts it into the display model
	 * 
	 * @param fieldname
	 *            the name in the model
	 * @param componentOrHolder
	 *            may be a component or a component holder
	 * @return the component of the holder
	 */
	public Component putComponentIntoDisplayModel(Object fieldname, Object componentOrHolder) {
		ComponentHolder holder;
		if (componentOrHolder instanceof Component) {
			holder = new ComponentHolder<>((Component) componentOrHolder);
		} else if (componentOrHolder instanceof ComponentHolder) {
			holder = (ComponentHolder) componentOrHolder;
		} else {
			throw new IllegalStateException("no fitting class " + componentOrHolder);
		}
		getDisplayModel().put(fieldname, holder);
		return holder.getComponent();
	}

	/**
	 * Wraps a component into a holder that changes its visibility and puts it into the display model
	 * 
	 * @param fieldname
	 *            the name in the model
	 * @param component
	 *            the value of the field
	 * @return component
	 */
	public Component putVisibilityComponentIntoDisplayModel(String fieldname, Component component) {
		getDisplayModel().put(fieldname, ComponentHolder.createVisiblitiyHolder(component));
		return component;
	}

	public JFrame getFrame() {
		return frame;
	}

	public void setFrame(JFrame frame) {
		this.frame = frame;
	}
}
