package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.event.ActionEvent;

import net.sf.gluebooster.java.booster.basic.container.BoostedHashSet;
import net.sf.gluebooster.java.booster.basic.events.EventMulticaster;

/**
 * Refactor into BoostedHashSet
 * 
 * @author CBauer
 *
 * @param <Element>
 */
public class BoostedHashSetExtension<Element> extends BoostedHashSet<Element> {

	/**
	 * Listeners that should be informed.
	 */
	private EventMulticaster listeners = new EventMulticaster(this);

	/**
	 * Are the listeners to be informed when this set becomes empty.
	 */
	private boolean informListenerWhenEmpty = false;

	/**
	 * Should an exception be thrown if the last element is removed.
	 */
	private boolean throwExceptionWhenEmptied = false;

	public boolean isInformListenerWhenEmpty() {
		return informListenerWhenEmpty;
	}

	public EventMulticaster getListeners() {
		return listeners;
	}

	public void setInformListenerWhenEmpty(boolean informListenerWhenEmpty) {
		this.informListenerWhenEmpty = informListenerWhenEmpty;
	}


	/**
	 * Informs all listeners that a command has occured.
	 * 
	 * @param command
	 *            the event.
	 */
	private void informActionListeners(String command) {
		ActionEvent event = new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
				command);
		listeners.actionPerformed(event);
		// TODO refactor listeners, so that only the command is a parameter
	}

	/**
	 * Checks whether this set is empty. Informs the listeners if empty.
	 */
	private void checkEmpty() {
		if (informListenerWhenEmpty && isEmpty()) {
			informActionListeners("isEmpty");
		}

		if (throwExceptionWhenEmptied && isEmpty()) {
			throw new IllegalStateException("set must not be empty");
		}
	}

	@Override
	public boolean remove(Object o) {
		boolean result = super.remove(o);
		checkEmpty();
		return result;
	}

	@Override
	public void clear() {
		super.clear();
		checkEmpty();

	}

	public boolean isThrowExceptionWhenEmptied() {
		return throwExceptionWhenEmptied;
	}

	public void setThrowExceptionWhenEmptied(boolean throwExceptionWhenEmptied) {
		this.throwExceptionWhenEmptied = throwExceptionWhenEmptied;
	}

}
