package net.sf.gluebooster.demos.pojo.refactor;

import net.sf.gluebooster.java.booster.essentials.logging.LogBooster;
import net.sf.gluebooster.java.booster.essentials.meta.Configurable;
import net.sf.gluebooster.java.booster.essentials.meta.HasName;
import net.sf.gluebooster.java.booster.essentials.meta.ObjectDescription;
import net.sf.gluebooster.java.booster.essentials.utils.TextBoostUtils;

/**
 * Duplicate of Boosted Object with generic name and implementation (clone).
 * 
 * TODO add the Id class to the generics
 * 
 * @author CBauer
 *
 */
public class BoostedObject<Name, Implementation extends BoostedObject>
		implements
		Configurable<ObjectDescription>, HasName<Name>, HasId, Cloneable,
		Codable, HasAttributes {

	/**
	 * Default log that can be used by instances (of subclasses). TODO make it
	 * static, but put it in a weak hashmap with the class as key. The variable
	 * should be put in the LogBooster.class
	 */
	private LogBooster log;

	/**
	 * The id of this object
	 */
	private Object id;

	/**
	 * Are hashCode and equals to be done with the id
	 */
	private boolean compareById = false;

	/**
	 * The configuration of this (or another) object.
	 * 
	 */
	private ObjectDescription configuration;

	/**
	 * The name of this object
	 */
	private Name name;

	/**
	 * Attributes of this object.
	 */
	private ObjectAttributes attributes;


	/**
	 * Default constructor.
	 */
	public BoostedObject() {
		log = new LogBooster(getClass());
	}

	public BoostedObject(ObjectDescription configuration) {
		this.configuration = configuration;
	}

	/**
	 * The default log
	 * 
	 * @return log
	 */
	protected LogBooster getLog() {
		return log;
	}

	@Override
	public Object getId() {
		return id;
	}

	@Override
	public void setId(Object id) {
		this.id = id;
	}

	@Override
	public int hashCode() {
		if (compareById) {
			if (id == null)
				return 0;
			else
				return id.hashCode();
		} else
			return super.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (compareById && obj instanceof HasId) {
			if (id != null)
				return id.equals(((HasId) obj).getId());
			else
				return obj == this;
		} else
			return super.equals(obj);
	}

	public boolean isCompareById() {
		return compareById;
	}

	public void setCompareById(boolean compareById) {
		this.compareById = compareById;
	}

	public ObjectDescription getConfiguration() {
		return configuration;
	}

	/**
	 * @Getter configuration
	 */
	@Override
	public ObjectDescription getConfigurationDescription() {
		return configuration;
	}

	@Override
	public void setConfiguration(ObjectDescription configurationValues) {
		this.configuration = configurationValues;
	}

	@Override
	public Name getName() {
		return name;
	}

	@Override
	public void setName(Name name) {
		this.name = name;
	}

	@Override
	public String toString() {
		if (name != null)
			return name.toString();
		else
			return super.toString();
	}

	// end copy of boosted Object

	@Override
	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

	@Override
	public String asSourcecode(StringBuilder result, Object... parameters)
			throws Exception {
		String uid = Refactor.getTemporaryId();
		String classname = getClass().getSimpleName();
		String objectname = "the" + classname + uid;
		TextBoostUtils.append(result, "\n", classname, " ", objectname,
				"= new ", classname, "();\n");
		if (id != null && (id instanceof String)) {
			TextBoostUtils
					.append(result, objectname, ".setId(\"", id, "\");\n");
		}
		if (name != null && (name instanceof String)) {
			TextBoostUtils.append(result, objectname, ".setName(\"", name,
					"\");\n");
		}

		return objectname;
	}

	/**
	 * Clones the id and name (and other necessary properties of subclasses).
	 * 
	 * @return the clone
	 */
	public Implementation cloneMeOnlyName()
			throws Exception {
		Implementation result = (Implementation) getClass().newInstance();
		result.setId(id);
		result.setName(name);
		return result;
	}


	/**
	 * Adds an object (plus prefix and suffix to a string builder if the object is not null
	 * 
	 * @param prefix
	 *            the prefix before the object
	 * @param object
	 *            the object
	 * @param suffix
	 *            the suffix after the object
	 * @param result
	 *            added are prefix, obejct, suffix
	 */
	protected void addIfNotNull(String prefix, Object object, String suffix,
			StringBuilder result) {
		if (object != null) {
			if (prefix == null)
				prefix = "";
			if (suffix == null)
				suffix = "";
			result.append(prefix).append(object).append(suffix);
		}
	}

	@Override
	public void setAttribute(Object key, Object value) throws Exception {
		attributes.setAttribute(key, value);

	}

	@Override
	public <Result> Result getAttribute(Object key) throws Exception {
		return (Result) attributes.getAttribute(key);
	}

	public ObjectAttributes getAttributes() {
		return attributes;
	}

	public void setAttributes(ObjectAttributes attributes) {
		this.attributes = attributes;
	}

}
