package net.sf.gluebooster.demos.pojo.refactor;

import java.util.Hashtable;

import javax.naming.CompositeName;
import javax.naming.Context;
import javax.naming.InvalidNameException;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;

import net.sf.gluebooster.java.booster.essentials.objects.BoostedObject;
import net.sf.gluebooster.java.booster.essentials.utils.ThrowableBoostUtils;

import org.apache.commons.collections15.Transformer;

/**
 * An extension of the ObjectFactory and other transformation classes.
 * 
 * @author CBauer
 * 
 * @param <Input>
 *            the input of the transformation
 * @param <Output>
 *            the output of the transformation
 */
public abstract class BoostedObjectFactory<Input,Output> extends BoostedObject implements ObjectFactory, Transformer<Input, Output> {

	/**
	 * Prevents a recursion when transforming objects.
	 */
	private static CompositeName FROM_TRANSFORM;
	
	static {
		try {
			FROM_TRANSFORM = new CompositeName("fromTransform");
		} catch (InvalidNameException ex){
			ex.printStackTrace();
		}
	}
	
	
	public BoostedObjectFactory() {
	}

	/**
	 * 
	 * @param name
	 *            the name of this factory.
	 */
	public BoostedObjectFactory(Object name) {
		setName(name);
	}

	@Override
	public Output transform(Input input) {
		try {
			return transformWithException(input);
		} catch (Exception ex){
			throw ThrowableBoostUtils.toRuntimeException(ex);
		}
	}

	/**
	 * Transforms but may throw an exception
	 * 
	 * @param input
	 *            the input to be transformed
	 * @return the transformation result
	 */
	@SuppressWarnings("unchecked")
	protected Output transformWithException(Input input) throws Exception {
		return (Output) getObjectInstance(input, FROM_TRANSFORM, null, null);
	}

	@SuppressWarnings("unchecked")
	@Override
	public Object getObjectInstance(Object input, Name name, Context nameCtx,
			Hashtable<?, ?> environment) throws Exception {
		if (FROM_TRANSFORM == name){
			throw new IllegalStateException("At least one of the following methods must be implemented: transform, getObjectInstance");
		} else {
			return transform((Input) input);
		}
	}

	
}
