package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.Component;
import java.awt.Label;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

/**
 * Contains and accesses a component.
 * 
 * @author cbauer
 *
 */
public class ComponentHolder<Value> implements HasValue<Value> {

	/**
	 * Access the value of the component
	 */
	private static final int TYPE_VALUE = 0;

	/**
	 * Access the visibility of the component
	 */
	private static final int TYPE_VISIBLE = 1;

	/**
	 * The type of the component access
	 */
	private int type = TYPE_VALUE;

	/**
	 * The held component
	 */
	private Component component;

	/**
	 * Constructor with component
	 * 
	 * @param component
	 *            the held component
	 */
	public ComponentHolder(Component component) {
		this.component = component;
	}

	private ComponentHolder(Component component, int type) {
		this(component);
		this.type = type;
	}

	/**
	 * Create a holder that changes the visibility of a component.
	 * 
	 * @param component
	 *            the component to show/hide.
	 * @return the created holder
	 */
	public static ComponentHolder createVisiblitiyHolder(Component component) {
		return new ComponentHolder<>(component, TYPE_VISIBLE);
	}

	@Override
	public Value getValue() {

		if (TYPE_VALUE == type) {
			if (component instanceof Label) {
				return (Value) ((Label) component).getText();
			} else if (component instanceof JLabel) {
				return (Value) ((JLabel) component).getText();
			} else if (component instanceof JTextComponent) {
				return (Value) ((JTextComponent) component).getText();
			} else if (component instanceof JCheckBox) {
				return (Value) Boolean.valueOf(((JCheckBox) component).isSelected());
			} else {
				throw new IllegalStateException("not yet supported " + component.getClass());
			}
		} else if (TYPE_VISIBLE == type) {
			return (Value) Boolean.valueOf(component.isVisible());
		} else {
			throw new IllegalStateException("type not supported " + type);
		}
	}

	@Override
	public void setValue(Value newValue) {

		if (TYPE_VALUE == type) {

			String stringValue = null;
			if (newValue != null) {
				stringValue = newValue.toString();
			}
			if (component instanceof Label) {
				((Label) component).setText(stringValue);
			} else if (component instanceof JLabel) {
				((JLabel) component).setText(stringValue);
			} else if (component instanceof JTextComponent) {
				((JTextComponent) component).setText(stringValue);
			} else if (component instanceof JCheckBox) {
				Boolean bool = (Boolean) newValue;
				boolean b = (bool == null) ? false : bool.booleanValue();
				((JCheckBox) component).setSelected(b);
			} else {
				throw new IllegalStateException("not yet supported:" + component.getClass().getName());
			}
		} else if (TYPE_VISIBLE == type) {
			component.setVisible((Boolean) newValue);
		} else {
			throw new IllegalStateException("type not supported " + type);
		}

	}

	public Component getComponent() {
		return component;
	}

	public void setComponent(Component component) {
		this.component = component;
	}

}
