package net.sf.gluebooster.demos.pojo.refactor;

import java.util.HashSet;
import java.util.Set;

import net.sf.gluebooster.java.booster.essentials.meta.HasName;

/**
 * Abstract implementation of a condition with some methods already implemented.
 * 
 * @author CBauer
 * 
 * @param <Implementation>
 *            The concrete subclass
 * @param <NameOfObjects>
 *            Class of the names (probably String)
 */
public abstract class ConditionBasics<Implementation extends BoostedObject<NameOfObjects, Implementation> & Condition<Implementation, NameOfObjects>, NameOfObjects>
		extends BoostedObject<NameOfObjects, Implementation>
		implements Condition<Implementation, NameOfObjects> {


	@Override
	public Boolean implies(Condition condition2) throws Exception {
		if (condition2 ==  null){
			return Boolean.TRUE;
		} else if (getClass().isInstance(condition2)) {
			return impliesImplementation((Implementation) condition2);
		} else {
			return Boolean.FALSE;
		}

	}

	/**
	 * Subclasses must overwrite this method with the concrete implication logic
	 * if implications should be possible.
	 * 
	 * @param condition2
	 *            the implication
	 * @return true if this implies the condition2
	 */
	protected Boolean impliesImplementation(Implementation condition2)
			throws Exception {
		throw new UnsupportedOperationException(
				"this method must be overwritten to have effects");
	}

	@Override
	public boolean isElementary() {
		throw new UnsupportedOperationException(
				"this method must be overwritten to have effects");
	}

	@Override
	public Condition createEmptyCondition() {
		throw new UnsupportedOperationException(
				"this method must be overwritten to have effects");
	}

	/**
	 * Just puts this element into a set. This method must be overwritten if
	 * anything other should take place.
	 */
	@Override
	public <Result extends Condition> Set<Result> split() {
		HashSet<Condition> result = new HashSet<Condition>(1);
		result.add(this);
		return null;
	}

	/**
	 * Replaces this name, if this is the object
	 */
	@Override
	public void replaceObjectName(HasName<NameOfObjects> object,
			NameOfObjects newName)
			throws Exception {
		if (this.equals(object)) {
			setName(newName);
		}

	}

	@Override
	public Boolean isInConflict(Condition condition) throws Exception {
		throw new IllegalStateException("not yet implemented in "
				+ getClass().getSimpleName());
	}

	@Override
	public Condition difference(Implementation condition) {
		throw new IllegalStateException("not yet implemented");
	}

	@Override
	public void remove(Condition condition) {
		throw new IllegalStateException("not yet implemented");

	}

	/**
	 * Adds more constraints
	 * 
	 * @param condition
	 *            the additional constraints
	 */
	protected void addFrom(Implementation condition) throws Exception {
		throw new UnsupportedOperationException(
				"this method must be overwritten to have effects");
	}

	@Override
	public void add(Condition condition) throws Exception {
		if (getClass().isInstance(condition)) {
			addFrom((Implementation) condition);
			return;
		}

		throw new UnsupportedOperationException(
				"this method must be overwritten to have effects");

	}


	/**
	 * Ist condition1 in conflict with condition2
	 * 
	 * @param condition1
	 *            the first condition
	 * @param condition2
	 *            the second condition
	 * @return true if there is a conflict
	 */
	public static boolean isInConflict(Condition condition1,
			Condition condition2) throws Exception {
		if (condition1 == null || condition2 == null)
			return false;
		else
			return condition1.isInConflict(condition2);
	}

	@Override
	public Implementation cloneMe() throws Exception {
		Implementation result = (Implementation) clone();
		return result;
	}


}
