package net.sf.gluebooster.demos.pojo.refactor;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collection;
import java.util.Map;

import net.sf.gluebooster.java.booster.essentials.utils.IoBoostUtils;
import net.sf.gluebooster.java.booster.essentials.utils.XmlBoostUtils;

/**
 * Utilitites for configurations.
 * 
 * @author cbauer
 *
 */
public class ConfigurationBoostUtils {

	/**
	 * Saves a configuration.
	 * 
	 * @param entries
	 *            a subset of the entries will be saved
	 * @param keys
	 *            the keys of the entries that are to be saved
	 * @param output
	 *            the configuration will be written into the output
	 * @param closeAfterSaving
	 *            is the output to be closed after saving.
	 */
	public static void saveConfiguration(Map<Object, Object> entries, Collection<Object> keys, OutputStream output, boolean closeAfterSaving) throws Exception {

		Map<Object, Object> configuration = ContainerBoostUtils.createSubmap(entries, keys, false);

		output.write("<?xml version='1.0' encoding='UTF-8'?><!-- Configuration Version: 1 -->\n".getBytes(IoBoostUtils.UTF_8));
		XmlBoostUtils.writeAsUtf8Xml(configuration, output, false, 0);
		// XMLEncoder encoder = new XMLEncoder(output);
		// encoder.writeObject(configuration);
		// if (closeAfterSaving) {
		// encoder.close();
		// }

		//
		// XMLConfiguration result = new XMLConfiguration();
		// for (Object key : keys) {
		// String name;
		// if (key instanceof HasName) {
		// name = ((HasName) key).getName().toString();
		// } else {
		// name = key.toString();
		// }
		// Object value = entries.get(key);
		// value = "noch nicht gesetzt";
		// result.setProperty(name, value);
		// }
		//
		// return result;

	}

	/**
	 * Saves a configuration.
	 * 
	 * @param entries
	 *            all the entries will be saved
	 * @param output
	 *            the configuration will be written into the output
	 * @param closeAfterSaving
	 *            is the output to be closed after saving.
	 */
	public static void saveConfiguration(Map<Object, Object> entries, OutputStream output, boolean closeAfterSaving) throws Exception {
		saveConfiguration(entries, entries.keySet(), output, closeAfterSaving);
	}

	/**
	 * Version of the load and save must be considered if the file format does change.
	 * 
	 * @param input
	 *            contains the previously saved data
	 * @param closeAfterReading
	 *            is the input to be closed after the reading
	 * @return the created configuration (map).
	 */
	public static <Result> Result loadConfiguration(InputStream input, boolean closeAfterReading) throws Exception {

		return (Result) XmlBoostUtils.readFromXml(input);
		// close after reading done automatically
		// XMLDecoder decoder = new XMLDecoder(input);
		// Object result = decoder.readObject();
		// if (closeAfterReading) {
		// decoder.close();
		// }
		// return result;
	}
}
