package net.sf.gluebooster.demos.pojo.refactor;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import net.sf.gluebooster.java.booster.essentials.logging.LogBooster;

/**
 * Refactor into net.sf.gluebooster.java.booster.essentials.utils.ContainerBoostUtils
 * 
 * @author cbauer
 *
 */
public class ContainerBoostUtils {

	/**
	 * The log of this class.
	 */
	private static LogBooster log = new LogBooster(ContainerBoostUtils.class);

	/**
	 * Copy parts of a source map into a target map
	 * 
	 * @param source
	 *            the source map
	 * @param target
	 *            the target map
	 * @param insertNullIfKeyNotFound
	 *            should a null value be inserted if the key is not found in the source map
	 * @param keys
	 *            the keys of the entries to be transfered from the source map into the target map.
	 */
	public static void copyVararg(Map source, Map target, boolean insertNullIfKeyNotFound, Object... keys) {
		copy(source, target, Arrays.asList(keys), insertNullIfKeyNotFound);
	}

	/**
	 * Copy parts of a source map into a target map
	 * 
	 * @param source
	 *            the source map
	 * @param target
	 *            the target map
	 * @param insertNullIfKeyNotFound
	 *            should a null value be inserted if the key is not found in the source map
	 * @param keys
	 *            the keys of the entries to be transfered from the source map into the target map.
	 */
	public static void copy(Map source, Map target, Collection<Object> keys, boolean insertNullIfKeyNotFound) {
		for (Object key : keys) {
			if (source.containsKey(key)) {
				target.put(key, source.get(key));
			} else {
				if (insertNullIfKeyNotFound) {
					target.put(key, null);
				} else {
					log.debug("no key:" + key);
				}

			}
		}
	}

	/**
	 * 
	 * Create a map with parts of a source map
	 * 
	 * @param source
	 *            the source map
	 * @param insertNullIfKeyNotFound
	 *            should a null value be inserted if the key is not found in the source map
	 * @param keys
	 *            the keys of the entries to be transfered from the source map into the target map.
	 * @return the created map
	 */
	public static Map<Object, Object> createSubmap(Map source, Collection<Object> keys, boolean insertNullIfKeyNotFound) throws Exception {
		HashMap<Object, Object> result = new HashMap<>();
		ContainerBoostUtils.copy(source, result, keys, insertNullIfKeyNotFound);
		return result;
	}

}
