package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Default actions that can happen on components or other pojos.
 * 
 * @author cbauer
 *
 */
public class DefaultActions implements ActionListener {

	/**
	 * The type to hide a component
	 */
	private static final int TYPE_SET_INVISIBLE = 1;

	/**
	 * The type of the action.
	 */
	private int type;

	/**
	 * The target of the action
	 */
	private Object target;

	/**
	 * Create a listener that sets a component to invisible when invoked.
	 * 
	 * @param targetToSetInvisible
	 *            the component to modify
	 * @return the created action listener
	 */
	public static DefaultActions createSetInvisibleAction(Component targetToSetInvisible) {
		return new DefaultActions(TYPE_SET_INVISIBLE, targetToSetInvisible);
	}

	private DefaultActions(int type) {
		this.type = type;
	}

	private DefaultActions(int type, Object target) {
		this(type);
		setTarget(target);
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		switch (type) {
		case TYPE_SET_INVISIBLE:
			((Component) target).setVisible(false);
			break;
		default:
			throw new IllegalStateException("type not supported: " + type);
		}

	}

	public int getType() {
		return type;
	}

	private void setType(int type) {
		this.type = type;
	}

	public Object getTarget() {
		return target;
	}

	public void setTarget(Object target) {
		this.target = target;
	}

}
