package net.sf.gluebooster.demos.pojo.refactor;

/**
 * Configuration of a dialog.
 * 
 * @author cbauer
 *
 */
public class DialogConfiguration {

	/**
	 * The type to choose a file and not a directory
	 */
	public static final int TYPE_CHOOSE_FILE = -1;

	/**
	 * The type to choose a directory and not a file
	 */
	public static final int TYPE_CHOOSE_DIRECTORY = -2;

	/**
	 * The parent (component) of the dialog.
	 */
	private Object parent;

	/**
	 * The message of the dialog;
	 */
	private Object message;

	/**
	 * The title of the dialog
	 */
	private String title;

	/**
	 * The type of the options. Uses the types of the JOptionPane plus the types defined here
	 * 
	 * @see JOptionPane
	 */
	private int dialogType;

	/**
	 * The initial value of the dialog;
	 */
	private Object initialValue;

	/**
	 * should an exception be thrown when nothing is entered/found or the dialog is cancelled.
	 */
	private boolean throwExceptionIfNothingSelected;

	/**
	 * Must the result of the dialog exist (not be null, existing file).
	 */
	private boolean resultMustExist;

	/**
	 * Should the initial value be returned if the dialog is cancelled.
	 */
	private boolean returnValueIfCancelled = false;

	/**
	 * 
	 * @param optionType
	 *            the type of the dialog according to the JOptionDialog
	 */
	public DialogConfiguration(Object parent, Object message, String title, int optionType) {
		super();
		this.parent = parent;
		this.message = message;
		this.title = title;
		this.dialogType = optionType;
	}

	/**
	 * 
	 * @param value
	 *            the initial value
	 */
	public DialogConfiguration(Object message, Object value) {
		super();
		this.message = message;
		this.initialValue = value;
	}

	/**
	 * 
	 * @param value
	 *            the initial value
	 * @param throwExceptionIfCancelled
	 *            should an exception be thrown if the dialog is cancelled.
	 */
	public DialogConfiguration(String title, int dialogType, Object value, boolean throwExceptionIfCancelled) {
		super();
		this.title = title;
		this.dialogType = dialogType;
		this.initialValue = value;
		this.throwExceptionIfNothingSelected = throwExceptionIfCancelled;
	}

	public Object getParent() {
		return parent;
	}

	public void setParent(Object parent) {
		this.parent = parent;
	}

	public Object getMessage() {
		return message;
	}

	public void setMessage(Object message) {
		this.message = message;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public int getDialogType() {
		return dialogType;
	}

	public void setDialogType(int optionType) {
		this.dialogType = optionType;
	}

	public Object getInitialValue() {
		return initialValue;
	}

	public void setInitialValue(Object value) {
		this.initialValue = value;
	}

	public boolean isThrowExceptionIfNothingSelected() {
		return throwExceptionIfNothingSelected;
	}

	public void setThrowExceptionIfNothingSelected(boolean throwExceptionIfCancelled) {
		this.throwExceptionIfNothingSelected = throwExceptionIfCancelled;
	}

	public boolean isResultMustExist() {
		return resultMustExist;
	}

	public void setResultMustExist(boolean resultMustExist) {
		this.resultMustExist = resultMustExist;
	}

	public boolean isReturnValueIfCancelled() {
		return returnValueIfCancelled;
	}

	public void setReturnValueIfCancelled(boolean returnValueIfCancelled) {
		this.returnValueIfCancelled = returnValueIfCancelled;
	}
	
	
}
