package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.Point;
import java.awt.Rectangle;

/**
 * Configuration for the display of some content.
 * 
 * @author CBauer
 * 
 * @param <Content>
 *            the displayed content.
 */
public class DisplayConfiguration<Content> {

	/**
	 * The title of the content.
	 */
	private String title;

	/**
	 * The position of the frame.
	 */
	private Point framePosition;

	/**
	 * The preferred size of the content
	 */
	private Rectangle preferredSize;

	/**
	 * The content.
	 */
	private Content[] content;

	public DisplayConfiguration() {

	}

	public DisplayConfiguration(String title, Content... content) {
		this.title = title;
		this.content = content;
	}

	public DisplayConfiguration(String title, Point framePosition,
			Rectangle preferredSize, Content... content) {
		this.title = title;
		this.framePosition = framePosition;
		this.preferredSize = preferredSize;
		this.content = content;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public Point getFramePosition() {
		return framePosition;
	}

	public void setFramePosition(Point framePosition) {
		this.framePosition = framePosition;
	}

	public Rectangle getPreferredSize() {
		return preferredSize;
	}

	public void setPreferredSize(Rectangle preferredSize) {
		this.preferredSize = preferredSize;
	}

	/**
	 * 
	 * @return Object[] because it is not ensured that it is a Content[] (when created with the varargs constructor).
	 */
	public Object[] getContent() {
		return content;
	}

	public void setContent(Content... content) {
		this.content = content;
	}

}
