package net.sf.gluebooster.demos.pojo.refactor;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.sf.gluebooster.java.booster.basic.events.BoostedChangeEvent;
import net.sf.gluebooster.java.booster.basic.events.HasChangeListeners;
import net.sf.gluebooster.java.booster.basic.io.IsCloseable;
import net.sf.gluebooster.java.booster.basic.meta.ObjectAttributes;
import net.sf.gluebooster.java.booster.essentials.utils.GeneralConstants;

/**
 * Adapted duplicate of net.sf.gluebooster.java.booster.basic.events
 * 
 * 
 * TODO make collections of the sets and create getter and setters. Check that the getter is no longer a new list.
 * 
 * Propagates events to multiple listeners.
 * 
 * @defaultParamText listener the specified listener
 * @author CBauer
 * 
 */
public class EventMulticaster extends BoostedObject implements IsCloseable, ChangeListener,
		PropertyChangeListener, HasChangeListeners, ActionListener// ,
																	// HasPropertyChangeListener
{
	  
	/**
	 * The source of the events.
	 */
  private Object eventSource;

	/**
	 * Is this multicaster closed and will not propagate any more events.
	 */
  private boolean closed = false;

	/**
	 * Listeners of property change events.
	 */
	private HashSet<PropertyChangeListener> propertyChangeListener;
  
	/**
	 * Listeners of change events.
	 */
	private HashSet<ChangeListener> changeListener;
  
  	/**
	 * Listeners of action events.
	 */
	private HashSet<ActionListener> actionListener;

	/**
	 * Handler of commands.
	 */
	private HashSet<CommandHandler> commandHandler;

	/**
	 * The eventSource is not set.
	 */
  public EventMulticaster(){
	  
  }
  
  /**
   * Constructor with event source.
   */
  public EventMulticaster(Object eventSource){
		this.eventSource = eventSource;
  }

	/**
	 * Add a specified listener.
	 */
  public void addPropertyChangeListener(PropertyChangeListener listener)
  {
		if (propertyChangeListener == null) {
			propertyChangeListener = new HashSet<>();
		}
		propertyChangeListener.add(listener);
  }
  
  	/**
	 * Remove a specified listener.
	 * 
	 * @return true, when the listener has been added before.
	 */
  public boolean removePropertyChangeListener(PropertyChangeListener listener)
  {
		if (propertyChangeListener == null) {
			return false;
		} else {
			return propertyChangeListener.remove(listener);
		}
  }

	/**
	 * Add a specified listener.
	 * 
	 */
  @Override
public void addChangeListener(ChangeListener listener)
  {
		if (changeListener == null) {
			changeListener = new HashSet<ChangeListener>();
		}
    changeListener.add(listener);
  }
   
  	/**
	 * Remove a specified listener.
	 * 
	 * @return true, when the listener had been added before.
	 */
  @Override
public boolean removeChangeListener(ChangeListener listener)
  {
		if (changeListener == null) {
			return false;
		} else {
			return changeListener.remove(listener);
		}
  }
  
@Override
public Collection<ChangeListener> getChangeListeners() {
		if (changeListener == null) {
			return Collections.EMPTY_LIST;
		} else {
			return new ArrayList<ChangeListener>(changeListener);
		}
}

@Override
public void addChangeListener(Collection<ChangeListener> listener) {
		if (changeListener == null) {
			changeListener = new HashSet<ChangeListener>();
		}
	changeListener.addAll(listener);
	
}

@Override
public void propertyChange(PropertyChangeEvent event)
{
		if (propertyChangeListener != null) {
			for (PropertyChangeListener listener : propertyChangeListener) {
				listener.propertyChange(event);
			}
		}
}

@Override
public void stateChanged(ChangeEvent event)
{
		if (changeListener != null) {
			for (ChangeListener listener : changeListener)
				listener.stateChanged(event);
		}
}

	/**
	 * Inform the listeners that a state has changed.
	 */
	public void stateChanged(Object eventSource)
{
		stateChanged(new ChangeEvent(eventSource));
}

public Object getEventSource() {
	return eventSource;
}

public void setEventSource(Object eventSource) {
	this.eventSource = eventSource;
}

@Override
public void close() throws IOException {
	closed = true;
	ObjectAttributes<String, Object> attributes = new ObjectAttributes<String, Object>();
	attributes.setName(GeneralConstants.CLOSE);
	
	BoostedChangeEvent<ObjectAttributes<String, Object>> event = new BoostedChangeEvent<ObjectAttributes<String,Object>>(eventSource, attributes);
	stateChanged(event);
	
}

@Override
public boolean isClosed() {
	
	return closed;
}


	/**
	 * Add a specified listener.
	 * 
	 */
	public void addActionListener(ActionListener listener) {
		if (actionListener == null) {
			actionListener = new HashSet<ActionListener>();
		}
		actionListener.add(listener);
	}

	/**
	 * Remove a specified listener.
	 * 
	 * @return true, when the listener has been added before.
	 */
	public boolean removeActionListener(ActionListener listener) {
		if (actionListener == null) {
			return false;
		} else {
			return actionListener.remove(listener);
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		if (actionListener != null) {
			for (ActionListener listener : actionListener)
				listener.actionPerformed(event);
		}

		if (commandHandler != null) {
			for (CommandHandler handler : commandHandler) {
				try {
					handler.executeCommand(event.getActionCommand(), null);
					// TODO make it more flexible to add (part of) the event as parameter when configured
				} catch (Exception ex) {
					getLog().info(ex);
				}
			}
		}

	}

	/**
	 * Add a specified listener.
	 * 
	 * @param handler
	 *            the handler to be added
	 * 
	 */
	public void addCommandHandler(CommandHandler handler) {
		if (commandHandler == null) {
			commandHandler = new HashSet<CommandHandler>();
		}
		commandHandler.add(handler);
	}

}
