package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.KeyListener;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import net.sf.gluebooster.java.booster.essentials.logging.LogBooster;

import org.apache.commons.collections15.TransformerUtils;

import edu.uci.ics.jung.algorithms.layout.FRLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.algorithms.layout.StaticLayout;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.renderers.Renderer;

/**
 * Does a layout for jung graphs
 * 
 * @author CBauer
 *
 */
public class JungGraphLayouter<Vertex, Edge> /* extends BoostedObject */{

	/**
	 * Writes logging information.
	 */
	private LogBooster log = new LogBooster(getClass());

	/**
	 * The graph that is to be layouted.
	 */
	private Graph<Vertex, Edge> graph;

	/**
	 * The configuration of the layout.
	 */
	private GraphDisplayConfiguration configuration;

	/**
	 * The wanted dimension of the layout
	 */
	private Dimension layoutDimension;

	/**
	 * The actual dimension of the layout
	 */
	private Dimension computedLayoutDimension;
	// private Transformer<Vertex, Paint> vertexFillPaintTransformer;
	// private boolean displayVertexLabel = false;
	// private Transformer<Vertex, String> vertexLabelTransformer;
	// private boolean displayVertexTooltip = false;
	// private Transformer<Vertex, String> vertexToolTipTransformer;
	// private GraphMouseListener<Vertex> graphMouseListener;
	// private List<KeyListener> keyListeners = new ArrayList<KeyListener>();
	// private Transformer<Vertex, Shape> vertexShapeTransformer;
	/**
	 * if false an exception will occur if there is overlapping (of nodes)
	 */
	private Flag overlappingAllowed = new Flag(true, false);

	public JungGraphLayouter() {

	}

	/**
	 * Creata a layout of the graph.
	 * 
	 * @return the created layout
	 */
	private Layout<Vertex, Edge> createLayout() {

		// create a basic layout by using an existing layout

		FRLayout<Vertex, Edge> layout = new FRLayout<Vertex, Edge>(graph);
		// Layout<BoostedNode, String> layout = new CircleLayout<BoostedNode,
		// String>(
		// graph);
		// TODO create the graph of the FRLayout and then modify the positions
		// (by using a staticLayout) so that no overlapping occurs
		layout.setSize(layoutDimension);
		// layout.setForceMultiplier()
		// layout.setRepulsionRange(300);
		// layout.setRepulsionMultiplier(50);
		// layout.setAttractionMultiplier(0.01);

		if (configuration.getVertexShapeTransformer() == null) {
			return layout;
			// hopefully the layout is good enough
			// nothing more needs be done yet
			// Optimization: use the modification of the else part always. Use
			// the default vertex shape transformer to compute the shapes of the
			// vertices.
		} else {

			// Optimize the layout by removing overlapping vertices and edges

			layout.initialize();
			// Compute the shapes of the vertices
			
			Graph<Shape, Edge> graph1 = new UndirectedSparseGraph<Shape, Edge>();
			
			HashMap<Vertex, Rectangle> shapes = new HashMap<Vertex, Rectangle>();
			for (Vertex vertex : graph.getVertices()) {
				Point2D center = layout.transform(vertex);
				Shape shape = (Shape) configuration.getVertexShapeTransformer()
						.transform(vertex);
				Rectangle bounds = shape.getBounds();
				bounds.x = (int) center.getX();
				bounds.y = (int) center.getY();
				shapes.put(vertex, bounds);
				graph1.addVertex(bounds);
			}

			// compute the edges between the shapes
			for (Edge edge : graph.getEdges()) {
				edu.uci.ics.jung.graph.util.Pair<Vertex> endpoints = graph
						.getEndpoints(edge);
				graph1.addEdge(edge, shapes.get(endpoints.getFirst()),
						shapes.get(endpoints.getSecond()));
			}

			// reposition the shapes to avoid overlapping
			GeometryPositionConstraint<Vertex> constraints = new GeometryPositionConstraint<Vertex>();
			constraints.setMinimumDistance(30);
			// GeometryBoostUtils
			// .repositionWithoutOverlapping(graph1, constraints);
			GeometryBoostUtils.position(graph1, constraints);

			// transfer the shape positions to the vertices
			int maxX = 0;
			int maxY = 0;
			HashMap<Vertex, Point2D> positions = new HashMap<Vertex, Point2D>();
			for (Entry<Vertex, Rectangle> entry : shapes.entrySet()) {
				Rectangle position = entry.getValue();
				positions.put(entry.getKey(), position.getLocation());
				maxX = Math.max(maxX, position.x + position.width);
				maxY = Math.max(maxY, position.y + position.height);
			}

			GeometryBoostUtils.checkOverlappings(graph1, overlappingAllowed,
					true, true, false, constraints);

			// add margin
			maxX = maxX + 10;
			maxY = maxY + 10;

			computedLayoutDimension = new Dimension(maxX, maxY);

			// Vertex locations can be placed in a Map and then supplied to this
			// layout as follows: Transformer vertexLocations =
			// TransformerUtils.mapTransformer(map);
			StaticLayout<Vertex, Edge> result = new StaticLayout<Vertex, Edge>(
					graph, TransformerUtils.mapTransformer(positions),
					computedLayoutDimension);

			return result;
		}
	}

	/**
	 * Creates a layout component of the graph.
	 * 
	 * @return the created layout
	 */
	public Component createLayoutComponent() {

		// FRLayout<Vertex, Edge> layout = new FRLayout<Vertex, Edge>(graph);
		// layout.setSize(layoutDimension);
		Layout<Vertex, Edge> layout = createLayout();

		// The BasicVisualizationServer<V,E> is parameterized by the edge types
		// BasicVisualizationServer<BoostedNode, String> vv = new
		// BasicVisualizationServer<BoostedNode, String>(
		// layout);
		VisualizationViewer<Vertex, Edge> vv = new VisualizationViewer<Vertex, Edge>(layout);

		if (configuration.isDisplayVertexLabel()) {
			if (configuration.getVertexLabelTransformer() != null) {
				vv.getRenderContext().setVertexLabelTransformer(
						configuration.getVertexLabelTransformer());
			} else if (configuration.getVertexToolTipTransformer() != null) {
				vv.getRenderContext().setVertexLabelTransformer(
						configuration.getVertexToolTipTransformer());
			}
		}

		if (configuration.isDisplayVertexTooltip()) {
			if (configuration.getVertexToolTipTransformer() != null) {
				vv.setVertexToolTipTransformer(configuration
						.getVertexToolTipTransformer());
			} else if (configuration.getVertexLabelTransformer() != null) {
				vv.getRenderContext().setVertexLabelTransformer(
						configuration.getVertexLabelTransformer());
			}
		}

		vv.setPreferredSize(computedLayoutDimension);
		if (configuration.getVertexBackgroundTransformer() != null) {
			vv.getRenderContext().setVertexFillPaintTransformer(
					configuration.getVertexBackgroundTransformer());
		} else {
			vv.getRenderContext().setVertexFillPaintTransformer(
					new TransformerToConstant(null));

		}

		if (configuration.getClickListener() != null) {
			vv.addGraphMouseListener(configuration.getClickListener());
		}

		List<KeyListener> listeners = configuration.getKeyListeners();
		if (listeners != null) {
			for (KeyListener keylistener : listeners) {
				vv.addKeyListener(keylistener);
			}
		}

		if (configuration.getVertexShapeTransformer() != null) {
			vv.getRenderContext().setVertexShapeTransformer(
					configuration.getVertexShapeTransformer());
		}

		// vv.getRenderContext().setVertexLabelRenderer(vertexLabelRenderer);
		vv.getRenderer().getVertexLabelRenderer()
				.setPosition(Renderer.VertexLabel.Position.CNTR);

		return vv;
		// JViewport viewport = new JViewport();
		// viewport.setView(vv);
		// viewport.setAutoscrolls(true);
		// return viewport;
	}

	public Graph<Vertex, Edge> getGraph() {
		return graph;
	}

	public void setGraph(Graph<Vertex, Edge> graph) {
		this.graph = graph;
	}

	public Dimension getLayoutDimension() {
		return layoutDimension;
	}

	public void setLayoutDimension(Dimension layoutDimension) {
		this.layoutDimension = layoutDimension;
		this.computedLayoutDimension = layoutDimension;
	}

	// public Transformer<Vertex, Paint> getVertexFillPaintTransformer() {
	// return vertexFillPaintTransformer;
	// }
	//
	// public void setVertexFillPaintTransformer(
	// Transformer<Vertex, Paint> vertexFillPaintTransformer) {
	// this.vertexFillPaintTransformer = vertexFillPaintTransformer;
	// }

	// public Transformer<Vertex, String> getVertexToolTipTransformer() {
	// return vertexToolTipTransformer;
	// }
	//
	// public void setVertexToolTipTransformer(
	// Transformer<Vertex, String> vertexToolTipTransformer) {
	// this.vertexToolTipTransformer = vertexToolTipTransformer;
	// }

	// public GraphMouseListener<Vertex> getGraphMouseListener() {
	// return graphMouseListener;
	// }
	//
	// public void setGraphMouseListener(
	// GraphMouseListener<Vertex> graphMouseListener) {
	// this.graphMouseListener = graphMouseListener;
	// }

	// public List<KeyListener> getKeyListeners() {
	// return keyListeners;
	// }
	//
	// public void setKeyListeners(List<KeyListener> keyListeners) {
	// this.keyListeners = keyListeners;
	// }
	//
	// public void addKeyListener(KeyListener listener) {
	// keyListeners.add(listener);
	// }

	// public Transformer<Vertex, Shape> getVertexShapeTransformer() {
	// return vertexShapeTransformer;
	// }
	//
	// public void setVertexShapeTransformer(
	// Transformer<Vertex, Shape> vertexShapeTransformer) {
	// this.vertexShapeTransformer = vertexShapeTransformer;
	// }
	//
	public Flag isOverlappingAllowed() {
		return overlappingAllowed;
	}

	public void setOverlappingAllowed(Flag overlappingAllowed) {
		this.overlappingAllowed = overlappingAllowed;
	}

	// public Transformer<Vertex, String> getVertexLabelTransformer() {
	// return vertexLabelTransformer;
	// }
	//
	// public void setVertexLabelTransformer(
	// Transformer<Vertex, String> vertexLabelTransformer) {
	// this.vertexLabelTransformer = vertexLabelTransformer;
	// }
	//
	// public boolean isDisplayVertexLabel() {
	// return displayVertexLabel;
	// }
	//
	// public void setDisplayVertexLabel(boolean displayVertexLabel) {
	// this.displayVertexLabel = displayVertexLabel;
	// }
	//
	// public boolean isDisplayVertexTooltip() {
	// return displayVertexTooltip;
	// }
	//
	// public void setDisplayVertexTooltip(boolean displayVertexTooltip) {
	// this.displayVertexTooltip = displayVertexTooltip;
	// }





	public GraphDisplayConfiguration getConfiguration() {
		return configuration;
	}

	public void setConfiguration(GraphDisplayConfiguration configuration) {
		this.configuration = configuration;
	}

}