package net.sf.gluebooster.demos.pojo.refactor;

import java.rmi.server.UID;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import net.sf.gluebooster.java.booster.essentials.meta.HasName;

/**
 * The value is now generic
 * 
 * 
 * Attributes of an object.
 * 
 * @author CBauer
 *
 */
public class ObjectAttributes<Name, Type, Value> implements Cloneable,
		HasName<Name>, HasAttributes, HasId<String>, HasValue<Value>, HasType<Type>, Map<Object, Object> {


	/**
	 * Is an indirect access via HasValue preferred
	 */
	private boolean preferIndirect = false;

	/**
	 * Additional attributes.
	 */
	@SuppressWarnings("rawtypes")
	private Map attributes = new HashMap();

	public ObjectAttributes() {
		setId(new UID().toString());
	}

	@Override
	public Name getName() {
		return getAttribute(HasName.class);
	}

	@Override
	public void setName(Name newName) {
		setAttribute(HasName.class, newName);
	}

	@Override
	public Type getType() {
		return getAttribute(HasType.class);
	}

	@Override
	public void setType(Type newType) {
		setAttribute(HasType.class, newType);
	}

	@SuppressWarnings("rawtypes")
	public Map getAttributes() {
		return attributes;
	}

	/**
	 * Get a value from the attributes map.
	 * 
	 * @param key
	 *            the wanted key of the map
	 * @return the value of the attributes map
	 */
	@SuppressWarnings("unchecked")
	public <Result> Result getFromAttributes(Object key) {
		return (Result) attributes.get(key);
	}

	/**
	 * Sets the value of an attribute.
	 * 
	 * @param key
	 *            the key of the attribute
	 * @param value
	 *            the value of the attribute
	 */
	@Override
	public void setAttribute(Object key, Object value) {

		if (preferIndirect) {
			Object field = attributes.get(key);
			if (field != null && field instanceof HasValue) {
				((HasValue) field).setValue(value);
				value = field;
			}
		}

		attributes.put(key, value);
	}

	@Override
	public <Result> Result getAttribute(Object key) {
		Object result = attributes.get(key);
		if (result != null && preferIndirect && result instanceof HasValue) {
			result = ((HasValue) result).getValue();
		}
		return (Result) result;
	}

	@Override
	public Value getValue() {
		return getAttribute(HasValue.class);
	}

	@Override
	public void setValue(Value value) {
		setAttribute(HasValue.class, value);
	}

	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		// if (name != null) {
		// Object theName = name;
		// if (theName != null && (theName instanceof HasName))
		// theName = ((HasName) theName).getName();
		//
		// result.append("name: ").append(theName).append("\n");
		// }
		// result.append("id: ").append(id).append("\n");
		// if (type != null)
		// result.append("type: ").append(type).append("\n");
		// if (value != null)
		// result.append("value: ").append(value).append("\n");
		if (attributes != null && !attributes.isEmpty())
			result.append(attributes).append("\n");
		return result.toString();
	}

	/**
	 * @return ObjectAttributes
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public Object clone() throws CloneNotSupportedException {
		ObjectAttributes result = (ObjectAttributes) super
				.clone();
		result.attributes = new HashMap(attributes);
		return result;
	}

	@Override
	public int hashCode() {
		return getId().hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof ObjectAttributes))
			return false;
		else {
			ObjectAttributes<Name, Type, Value> object = (ObjectAttributes<Name, Type, Value>) obj;
			return object.getId() == getId();
		}
	}

	@Override
	public String getId() {
		return getAttribute(HasId.class);
	}

	/**
	 * The default id may be overwritten.
	 * 
	 * @param id
	 */
	@Override
	public void setId(String id) {
		setAttribute(HasId.class, id);
	}

	public boolean isPreferIndirect() {
		return preferIndirect;
	}

	public void setPreferIndirect(boolean preferIndirect) {
		this.preferIndirect = preferIndirect;
	}

	@Override
	public void clear() {
		if (preferIndirect) {
			for (Object key : attributes.keySet()) {
				setAttribute(key, null);
			}
		} else {
			attributes.clear();
		}
	}

	@Override
	public Object put(Object key, Object value) {
		Object result = getAttribute(key);
		setAttribute(key, value);
		return result;
	}

	@Override
	public void putAll(Map m) {
		if (preferIndirect) {
			for (Object key : m.keySet()) {
				setAttribute(key, m.get(key));
			}
		} else {
			attributes.putAll(m);
		}
	}

	@Override
	public Object remove(Object key) {
		return attributes.remove(key);
	}

	@Override
	public int size() {
		return attributes.size();
	}

	@Override
	public Collection values() {
		if (preferIndirect) {
			HashSet<Object> result = new HashSet<>();
			for (Object key : attributes.keySet()) {
				result.add(getAttribute(key));
			}
			return result;
		} else {
			return attributes.values();
		}
	}

	@Override
	public boolean containsKey(Object key) {
		return attributes.containsKey(key);
	}

	@Override
	public boolean containsValue(Object value) {
		return values().contains(value);
	}

	@Override
	public Set entrySet() {
		return attributes.entrySet();
	}

	@Override
	public Object get(Object key) {
		return getAttribute(key);
	}

	@Override
	public boolean isEmpty() {
		return attributes.isEmpty();
	}

	@Override
	public Set keySet() {
		return attributes.keySet();
	}
}
