package net.sf.gluebooster.demos.pojo.refactor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;

import net.sf.gluebooster.java.booster.basic.container.BoostedNode;

/**
 * Some algorithms to sorts PlanNodes
 * 
 * @author CBauer
 *
 */
public class OperationSorter implements ObjectFactory,
		Comparator<BoostedNode/* with Operation */> {

	/**
	 * Sort by the abstraction level of the operator (descending).
	 */
	private static final int SORT_BY_OPERATOR_ABSTRACTION_DESC = 1;

	/**
	 * The type of the sorting algorithm.
	 */
	private int type;

	/**
	 * Creates a sorter, that sorts by the abstraction level of the operator
	 * (descending).
	 * 
	 * @return the created sorter
	 */
	public static OperationSorter createSortByOperatorAbstractionDesc() {
		return new OperationSorter(SORT_BY_OPERATOR_ABSTRACTION_DESC);
	}

	private OperationSorter(int type) {
		this.type = type;
	}

	/**
	 * Sorts some plan nodes.
	 * 
	 * @param planNodes
	 *            Collection of PlanNode that is to be sorted
	 * @return List<PlanNode>
	 */
	@Override
	public Object getObjectInstance(Object planNodes, Name name,
			Context nameCtx, Hashtable<?, ?> environment) throws Exception {
		switch (type) {
		case SORT_BY_OPERATOR_ABSTRACTION_DESC:
			ArrayList<BoostedNode> result = new ArrayList(
					((Collection<BoostedNode>) planNodes));
			Collections.sort(result, this);
			return result;

		default:
			throw new IllegalStateException("Type not supported: " + type);
		}
	}

	@Override
	public int compare(BoostedNode node1, BoostedNode node2) {
		switch (type) {
		case SORT_BY_OPERATOR_ABSTRACTION_DESC:

			Operation operat1 = Operation.getOperation(node1);
			Integer level1 = operat1.getOperator()
					.getAbstractionLevel();
			Integer level2 = Operation.getOperation(node2).getOperator()
					.getAbstractionLevel();
			return level2.compareTo(level1); // descending

		default:
			throw new IllegalStateException("Type not supported: " + type);
		}
	}

}
