package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.beans.BeanInfo;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.event.ChangeListener;

import net.sf.gluebooster.java.booster.basic.container.BoostedNode;
import net.sf.gluebooster.java.booster.basic.container.BoostedNodeUtils;
import net.sf.gluebooster.java.booster.basic.container.SimpleBoostedNode;
import net.sf.gluebooster.java.booster.basic.gui.swing.SwingFactory;
import net.sf.gluebooster.java.booster.basic.meta.ObjectAnalyzer;
import net.sf.gluebooster.java.booster.essentials.utils.Check;
import net.sf.gluebooster.java.booster.essentials.utils.ContainerBoostUtils;
import net.sf.gluebooster.java.booster.essentials.utils.TextBoostUtils;
import net.sf.gluebooster.java.booster.essentials.utils.ThreadBoostUtils;

import org.apache.commons.collections15.MultiMap;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

import edu.uci.ics.jung.graph.DirectedSparseMultigraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.UndirectedGraph;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;

/**
 * Methods to be refactored later. The methods may be moved into other gluebooster artifacts.
 * 
 * TODO: Write more tests for the boostednode operations.
 * 
 * @author CBauer
 *
 * @defaultParamText outPoint the out-point of the node
 * @defaultParamText inPoint the in-point of the node
 * @defaultParamText predecessor the starting point of the (directed) edge
 * @defaultParamText successor the ending point of the (directed) edge
 * @defaultParamText graph the inspected graph
 * @defaultParamText node the inspected node
 *
 */
public class Refactor {

	// check the path to the jfxrt and if it is available in all java versions
	// move into basic pom
	// mvn install:install-file -Dfile="C:\Program Files\Java\jdk1.7.0_07\jre\lib\jfxrt.jar" -DgroupId=com.oracle -DartifactId=javafx -Dversion=2.0
	// -Dpackaging=jar

	// add to root pom:
	// svn can be installed using sliksvn
	// gpg is installed looking for GnuPG
	// After the installation the secret key is to be imported via gpg --allow-secret-key-import --import private.key

	/**
	 * Create a directed graph from a graph.
	 * 
	 * Refactor into JungGraphBoostUtils
	 * 
	 * @param graph
	 *            the original graph.
	 * @return the created graph
	 */
	public static Graph<BoostedNode, String> createDefaultDirectedGraph(
			BoostedNodeGraph graph) {

		DirectedSparseMultigraph<BoostedNode, String> result = new DirectedSparseMultigraph<>();

		Set<BoostedNode> nodes = graph.getAllVertices();

		int counter = 0;
		// Add the vertices
		for (BoostedNode node : nodes) {
			result.addVertex(node);
		}

		// predecessors are sufficient, otherwise all edges would be duplicated
		for (BoostedNode node : nodes) {
			for (BoostedNode predecessor : BoostedNodeUtils
					.getPredecessors(node)) {
				// result.addVertex(inPoint);
				result.addEdge("edge-" + counter++ /*
													 * + " from " + predecessor
													 * + " to " + node
													 */, predecessor, node);
			}
		}


		return result;
	}


	/**
	 * Refactor into JungGraphBoostUtils. Creates a graphical display of a given
	 * graph.
	 * 
	 * @param boostedNodeGraph
	 *            the graph
	 * @param displayConfiguration
	 *            the configuration of the display
	 * @param overlappingAllowed
	 *            may the display have overlapping nodes
	 * @return the display
	 */
	public static <NameOfObjects> Component createOverviewGraphComponent(
			BoostedNodeGraph boostedNodeGraph,
			// final int vertexDetailDisplayLevel,
			/* final boolean displayVertexValueInsteadOfName, */
			final GraphDisplayConfiguration<BoostedNode, NameOfObjects> displayConfiguration,
			Flag overlappingAllowed) {
		Graph<BoostedNode, String> graph = createDefaultDirectedGraph(boostedNodeGraph);




		JungGraphLayouter layouter = new JungGraphLayouter<BoostedNode, String>();
		layouter.setConfiguration(displayConfiguration);
		layouter.setGraph(graph);
		layouter.setLayoutDimension(new Dimension(300, 300));
		
		layouter.setOverlappingAllowed(overlappingAllowed);
		return layouter.createLayoutComponent();
	}

	/**
	 * Refactor into BoostedNodeUtils. A sink vertex in a directed graph is a
	 * vertex without outgoing edge. See
	 * http://en.wikipedia.org/wiki/Vertex_%28graph_theory%29
	 * 
	 * @param nodeOfGraph
	 *            the starting point
	 * @return the found sinks
	 */
	public static Set<BoostedNode> getAllSinkVertices(
			final BoostedNode nodeOfGraph) {
		HashSet<BoostedNode> result = new HashSet<BoostedNode>();
		for (BoostedNode node : BoostedNodeUtils
				.getAllNodesOfGraph(nodeOfGraph)) {
			if (BoostedNodeUtils.getSuccessors(node).isEmpty()) {
				result.add(node);
			}
		}
		return result;
	}

	/**
	 * Refactor into BoostedNodeUtils. A source vertex in a directed graph is a
	 * vertex without ingoing edge. See
	 * http://en.wikipedia.org/wiki/Vertex_%28graph_theory%29
	 * 
	 * @param nodeOfGraph
	 *            the starting point
	 * @return the found sources
	 */
	public static Set<BoostedNode> getAllSourceVertices(
			final BoostedNode nodeOfGraph) {
		HashSet<BoostedNode> result = new HashSet<BoostedNode>();
		for (BoostedNode node : BoostedNodeUtils
				.getAllNodesOfGraph(nodeOfGraph)) {
			if (BoostedNodeUtils.getPredecessors(node).isEmpty()) {
				result.add(node);
			}
		}
		return result;
	}

	/**
	 * Creates a predecessor node for a given node.
	 * 
	 * Refactor into boostednodeutils
	 * 
	 * @param node
	 *            the starting point
	 * @param listener
	 *            will be added to the predecessor
	 * @return the created node
	 */
	public static BoostedNode createPredecessor(BoostedNode node,
			ChangeListener listener) throws Exception {
		BoostedNode predecessorNode = node.createPeer();

		addEdge(predecessorNode, node);

		if (listener != null)
			predecessorNode.addChangeListener(listener);

		// checkPredecessorSuccessor(node);
		// checkPredecessorSuccessor(predecessorNode);

		return predecessorNode;
	}

	/**
	 * Add an edge to the graph
	 * 
	 * @param predecessor
	 *            the starting point of the edge
	 * @param successor
	 *            the end point of the edge
	 */
	public static void addEdge(BoostedNode predecessor, BoostedNode successor)
			throws Exception {
		BoostedNode outPoint = addOutPoint(predecessor);
		addEdgeFromOutpointToNode(predecessor, outPoint, successor);

	}

	/**
	 * Creates a new inPoint in the successor and connects the outPoint with the inPoint
	 * 
	 * @param outPoint
	 *            the start of the edge
	 * @param successor
	 *            contains the outpoint
	 */
	public static void addEdgeFromOutpointToNode(BoostedNode predecessor,
			BoostedNode outPoint, BoostedNode successor) throws Exception {
		BoostedNode inPoint = addInPoint(successor);
		addEdgeFromOutpointToInpoint(predecessor, outPoint, inPoint, successor);

	}

	/**
	 * Adds an edge from a node to a given in-point
	 * 
	 * @param predecessor
	 *            the start of the edge
	 * @param inPoint
	 *            the end-point of the edge
	 * @param successor
	 *            contains the inPoint
	 */
	public static void addEdgeFromNodeToInpoint(BoostedNode predecessor,
			BoostedNode inPoint, BoostedNode successor) throws Exception {
		BoostedNode outPoint = addOutPoint(predecessor);
		addEdgeFromOutpointToInpoint(predecessor, outPoint, inPoint, successor);

	}

	/**
	 * Add a edge from a give out-point to a given in-point. TODO is it possible to get the predecessor and successor from the point?
	 * 
	 * @param predecessor
	 *            contains the out-point
	 * @param outPoint
	 *            start of the edge
	 * @param inPoint
	 *            end of the edge
	 * @param successor
	 *            contains the in-point.
	 */
	public static void addEdgeFromOutpointToInpoint(BoostedNode predecessor,
			BoostedNode outPoint, BoostedNode inPoint, BoostedNode successor)
			throws Exception {

		predecessor.addOutgoingEdge(outPoint, inPoint);
		successor.addIngoingEdge(outPoint, inPoint);

		// checkPredecessorSuccessor(predecessor);
		// checkPredecessorSuccessor(successor);
	}

	/**
	 * Creates a new inPoint in the successor and connects the outPoint with the inPoint
	 * 
	 * @param predecessor
	 *            contains the out-point
	 * @param outPoint
	 *            from this point the edges will start
	 * @param successors
	 *            the ends of the edges
	 */
	public static void addEdgeFromOutpointToNodes(BoostedNode predecessor,
			BoostedNode outPoint, Collection<BoostedNode> successors)
			throws Exception {
		for (BoostedNode successor : successors) {
			addEdgeFromOutpointToNode(predecessor, outPoint, successor);
		}
	}

	/**
	 * Add multiple edges from some nodes to another node
	 * 
	 * @param predecessors
	 *            the edges will start there
	 * @param inPoint
	 *            the edges will end in this point
	 * @param successor
	 *            contains the in-point
	 */
	public static void addEdgeFromNodesToInpoint(
			Collection<BoostedNode> predecessors, BoostedNode inPoint,
			BoostedNode successor) throws Exception {
		for (BoostedNode predecessor : predecessors) {
			addEdgeFromNodeToInpoint(predecessor, inPoint, successor);
		}
	}

	/**
	 * Refactor into boostednodeutils. Create a success node with listener.
	 * 
	 * 
	 * @param listener
	 *            listener of the new node
	 * @return the created node
	 */
	public static BoostedNode createSuccessor(BoostedNode node,
			ChangeListener listener) throws Exception {
		BoostedNode successorNode = node.createPeer();

		BoostedNode inPoint = addInPoint(successorNode);

		BoostedNode outPoint = addOutPoint(node);

		successorNode.addIngoingEdge(outPoint, inPoint);
		node.addOutgoingEdge(outPoint, inPoint);

		if (listener != null)
			successorNode.addChangeListener(listener);

		// checkPredecessorSuccessor(node);
		// checkPredecessorSuccessor(successorNode);

		return successorNode;
	}

	/**
	 * Hack because of bug of BoostedNodeUtils
	 * 
	 * @return the out-point
	 * @throws Exception
	 */
	public static BoostedNode addOutPoint(BoostedNode node) throws Exception {

		// TODO BoostedNode.createOutPoint : modify the comment, that the
		// created point is automatically added to the out points
		// TODO modify the SimpleBoostedNode accordingly
		// TODO modify this method (and addInPoint) accordingly
		// TODO modify BoostedNode: add the setter for the outpoints

		ArrayList<BoostedNode> outPoints = new ArrayList<BoostedNode>(
				node.getOutPoints());

		BoostedNode outPoint = node.createOutPoint();
		outPoint.setZoomOut(Pair.of(node, outPoint), null);

		// node.getOutPoints().add(outPoint);
		outPoints.add(outPoint);
		((SimpleBoostedNode) node).setOutPoints(outPoints);
		return outPoint;
	}

	/**
	 * Add a new in-point to a node
	 * 
	 * @return the created in-point
	 */
	public static BoostedNode addInPoint(BoostedNode node) throws Exception {

		ArrayList<BoostedNode> inPoints = new ArrayList<BoostedNode>(
				node.getInPoints());

		BoostedNode inPoint = node.createInPoint();
		inPoint.setZoomOut(Pair.of(node, inPoint), null);

		inPoints.add(inPoint);
		((SimpleBoostedNode) node).setInPoints(inPoints);
		return inPoint;
	}

	/**
	 * Create a predecessor node.
	 * 
	 * @return the created node
	 * @throws Exception
	 */
	public static BoostedNode createPredecessor(BoostedNode node)
			throws Exception {
		return createPredecessor(node, null);
	}

	/**
	 * Create a successor node
	 * 
	 * @return the created node
	 */
	public static BoostedNode createSuccessor(BoostedNode node)
			throws Exception {
		BoostedNode result = createSuccessor(node, null);

		return result;
	}

	/**
	 * Refactor into BoostedNodeUtils
	 * 
	 * Replaces the node by a subgraph. The nodes of the subgraph are changed.
	 * 
	 * TODO refactor this method and check the structure. remove the in and out
	 * point mappings. DO this in the original classes.
	 * 
	 * @param node
	 * @param replaceBy
	 *            may be empty or null, then the node is removed and its pre-
	 *            and successors are connected
	 */
	public static void replace(BoostedNode node, BoostedNodeGraph replaceBy)
			throws Exception {

		// if (replaceBy.getAllVertices().isEmpty()){
		// throw new IllegalStateException("replaceBy is empty");
		// }
		
		// checkPredecessorSuccessor(node);

		// Cache the predecessors and successors before they may be changed
		// during the following algorithm
		Collection<BoostedNode> nodePredecessors = BoostedNodeUtils
				.getPredecessors(node);
		Collection<BoostedNode> nodeSuccessors = BoostedNodeUtils
				.getSuccessors(node);

		boolean emptyReplacement = (replaceBy == null) || replaceBy.isEmpty();
		// if the replaceBy graph is empty the predecessors and successors of
		// the node must be connected directly.
		// To prevent that edges are duplicate count the edges when redirecting
		// the in-points and only insert additional edges of the out-points when
		// there are more

		Collection<BoostedNode> newSuccessors;
		if (emptyReplacement) {
			newSuccessors = nodeSuccessors;
		} else {
			newSuccessors = replaceBy.getAllSourceVertices();
		}

		Collection<BoostedNode> newPredecessors;
		if (emptyReplacement) {
			newPredecessors = Collections.EMPTY_SET;// because the connections have already been set // nodePredecessors;
		} else {
			newPredecessors = replaceBy.getAllSinkVertices();
		}

		CountingMap<Pair<BoostedNode, BoostedNode>> addedEdgeCount = new CountingMap<Pair<BoostedNode, BoostedNode>>();

		//redirect in-Points
		for (BoostedNode inPoint: node.getInPoints()){
			for (BoostedNode outPoint : new ArrayList<BoostedNode>(
					node.getPredecessorOutPoints(inPoint))) {
				// use a new list, because the list may change dynamically

				BoostedNode predecessorNode = outPoint.getZoomOut(null)
						.getKey();
				// checkPredecessorSuccessor(predecessorNode);

				removeEdge(predecessorNode, outPoint, inPoint, node);
				addEdgeFromOutpointToNodes(predecessorNode, outPoint,
						newSuccessors);

				// for (BoostedNode sourceVertex : newSuccessors) {
				//
				// List<BoostedNode> successorInPoints = predecessorNode
				// .getSuccessorInPoints(outPoint);
				// if (successorInPoints.isEmpty()) {
				//
				// throw new IllegalStateException(
				// "there should be an successor in-point");
				// }
				//
				// // System.out.println("Structure before "
				// // + getStructureString(node)
				// // + getStructureString(predecessorNode));
				//
				// successorInPoints.remove(inPoint);
				// BoostedNode newInPoint = addInPoint(sourceVertex);
				// successorInPoints.add(newInPoint);
				// sourceVertex.addIngoingEdge(outPoint, newInPoint);
				// addedEdgeCount
				// .increment(new ImmutablePair<BoostedNode, BoostedNode>(
				// predecessorNode, sourceVertex));
				// // System.out.println("Structure after "
				// // + getStructureString(node)
				// // + getStructureString(predecessorNode));
				// }
				// checkPredecessorSuccessor(predecessorNode);
			}
			// node.getInPoints().remove(inPoint);
			removeInPoint((SimpleBoostedNode) node, inPoint);
		}

		// checkPredecessorSuccessor(node);

		// redirect out-Points
		for (BoostedNode outPoint : node.getOutPoints()) {
			for (BoostedNode inPoint : new ArrayList<BoostedNode>(
					node.getSuccessorInPoints(outPoint))) {
				// use a new list, because the list may change dynamically

				BoostedNode successorNode = inPoint.getZoomOut(null).getKey();
				// checkPredecessorSuccessor(successorNode);

				removeEdge(node, outPoint, inPoint, successorNode);
				addEdgeFromNodesToInpoint(newPredecessors, inPoint,
						successorNode);

// for (BoostedNode sinkVertex : newPredecessors) {
				//
				// List<BoostedNode> predecessorOutPoints = successorNode
				// .getPredecessorOutPoints(inPoint);
				// if (predecessorOutPoints.isEmpty()) {
				// throw new IllegalStateException(
				// "there should be an predecessor out-point");
				// }
				// // System.out.println("Structure before "
				// // + getStructureString(node)
				// // + getStructureString(successorNode));
				//
				// ImmutablePair<BoostedNode, BoostedNode> newEdge = new
				// ImmutablePair<BoostedNode, BoostedNode>(
				// sinkVertex, successorNode);
				// addedEdgeCount.decrement(newEdge);
				// predecessorOutPoints.remove(outPoint);
				// if ((!emptyReplacement)
				// || addedEdgeCount.count(newEdge) < 0) {
				//
				// BoostedNode newOutPoint = addOutPoint(sinkVertex);
				// predecessorOutPoints.add(newOutPoint);
				// sinkVertex.addOutgoingEdge(newOutPoint, inPoint);
				// }
				// // System.out.println("Structure after"
				// // + getStructureString(node)
				// // + getStructureString(successorNode));
				// }
				// checkPredecessorSuccessor(successorNode);
			}
			removeOutPoint((SimpleBoostedNode) node, outPoint);
			// node.getOutPoints().remove(outPoint);
		}

		// checkPredecessorSuccessor(node);

	}
	
	/**
	 * Removes one edge between the specified points
	 * 
	 * @param predecessorOutpoint
	 *            the out-point of the predecessor
	 * @param successorInpoint
	 *            the in-point of the successor
	 * @param successor
	 *            the successor node
	 */
	public static void removeEdge(BoostedNode predecessor,
			BoostedNode predecessorOutpoint, BoostedNode successorInpoint,
			BoostedNode successor) {
		if (!predecessor.getSuccessorInPoints(predecessorOutpoint).remove(
				successorInpoint)) {
			throw new IllegalStateException(
					"There is no connection from the predecessorOutpoint  to the successorInpoint");
		}
		if (!successor.getPredecessorOutPoints(successorInpoint).remove(
				predecessorOutpoint)) {
			throw new IllegalStateException(
					"There is no connection to the successorInpoint from the predecessorOutpoint  ");
		}

	}

	/**
	 * Refactor into BoostedNode as toStructureString in SimpleBoostedNode. Display the structure of a node as string
	 * 
	 * @return out-points and in-points mappings
	 */
	public static String getStructureString(BoostedNode node) throws Exception {
		StringBuilder result = new StringBuilder();
		result.append(node.getAttributes().getName()).append("\n");
		result.append("out: ")
				.append(ObjectAnalyzer
						.getPrivateField(node, "outPointsMapping"))
				.append("\n");
		result.append("in: ")
				.append(ObjectAnalyzer.getPrivateField(node, "inPointsMapping"))
				.append("\n");
		return result.toString();
	}

	/**
	 * Checks that predecessors and successors are correctly connected to a node
	 */
	public static void checkPredecessorSuccessor(BoostedNode node) throws Exception{

		// check successors
		for (BoostedNode outPoint : node.getOutPoints()) {
			for (BoostedNode inPoint : node.getSuccessorInPoints(outPoint)) {
				Pair<BoostedNode, BoostedNode> successorAndFocus = inPoint
						.getZoomOut(null);
				BoostedNode successor = successorAndFocus.getLeft();
				if (!successor.getPredecessorOutPoints(inPoint).contains(
						outPoint)) {
					throw new IllegalStateException(
							"successor in-point has no connection to node out-point");
				}

			}
		}

		// check predecessors
		for (BoostedNode inPoint : node.getInPoints()) {
			for (BoostedNode outPoint : node.getPredecessorOutPoints(inPoint)) {
				Pair<BoostedNode, BoostedNode> predecessorAndFocus = outPoint
						.getZoomOut(null);
				BoostedNode predecessor = predecessorAndFocus.getLeft();
				if (!predecessor.getSuccessorInPoints(outPoint).contains(
						inPoint)) {
					throw new IllegalStateException(
							"predecessor out-point has no connection to node in-point");
				}

			}
		}
		
	}

	/**
	 * Removes one in-point from a given node.
	 */
	private static void removeInPoint(SimpleBoostedNode node,
			BoostedNode inPoint) {

		ArrayList<BoostedNode> newInPoints = new ArrayList<BoostedNode>(
				node.getInPoints());
		newInPoints.remove(inPoint);
		node.setInPoints(newInPoints);

	}

	/**
	 * Removes one out-point from a given node.
	 */
	private static void removeOutPoint(SimpleBoostedNode node,
			BoostedNode outPoint) {

		ArrayList<BoostedNode> newOutPoints = new ArrayList<BoostedNode>(
				node.getOutPoints());
		newOutPoints.remove(outPoint);
		node.setOutPoints(newOutPoints);

	}

	/**
	 * If the node has multiple edges to the same successor, only one will remain
	 * 
	 * @param node
	 *            the inspected node
	 */
	public static void removeDuplicateEdges(BoostedNode node) {
		List<BoostedNode> successors = BoostedNodeUtils.getSuccessors(node);
		HashSet<BoostedNode> successorSet = new HashSet<BoostedNode>(successors);
		if (successorSet.size() < successors.size()) {
			CountingMap counter = new CountingMap<BoostedNode>();
			for (BoostedNode successor : successors) {
				counter.increment(successor);
			}
			for (BoostedNode outpoint : node.getOutPoints()) {
				List<BoostedNode> successorInPoints = node
						.getSuccessorInPoints(outpoint);
				// iterator over a new list because otherwise a
				// ConcurrentModificationException can occur
				for (BoostedNode inpoint : new ArrayList<BoostedNode>(
						successorInPoints)) {
					BoostedNode successor = inpoint.getZoomOut(null).getLeft();
					counter.decrement(successor);
					if (counter.get(successor).longValue() > 0) {
						successorInPoints.remove(inpoint);
						successor.getPredecessorOutPoints(inpoint).remove(
								outpoint);
					}
				}
				
			}


		}

	}

	/**
	 * Refactor into reflectionBoostUtils. Sets the property of an object
	 * 
	 * @param object
	 *            contains the property
	 * @param propertyName
	 *            the name of the property
	 * @param value
	 *            the new value
	 */
	public static void setProperty(Object object, String propertyName,
			Object value) throws Exception {
		BeanInfo beanInfo = java.beans.Introspector.getBeanInfo(object
				.getClass());
		PropertyDescriptor[] descriptors = beanInfo.getPropertyDescriptors();
		for (PropertyDescriptor property : descriptors) {
			if (property.getName().equals(propertyName)) {
				Method writeMethod = property.getWriteMethod();
				if (writeMethod != null) {
					writeMethod.invoke(object, value);
				} else {
					throw new IllegalStateException(
							"no write method for property " + propertyName
									+ " of " + object);
				}
				return;
			}
		}

		throw new IllegalStateException("property '" + propertyName
				+ "' not found in " + object + " Available properties are "
				+ ContainerBoostUtils.toString(descriptors, "(", ", ", ")"));
	}

	/**
	 * Refactor into Check. Checks that an object is not null
	 * 
	 * @param object
	 *            the object to check
	 * @param name
	 *            the name of the object
	 * @param additionalInfo
	 *            used for debugging
	 * @return the object
	 * @throws RuntimeException
	 *             if null
	 */
	public static <Result> Result notNull(Result object, Object name,
			Object... additionalInfo) throws RuntimeException {
		Check.notNull(object, name, additionalInfo);
		return object;
	}

	/**
	 * Refactor into Check. Tests that a collection is not null
	 * 
	 * 
	 * @param collection
	 *            the collection to be inspected
	 * @param name
	 *            the name of the collection
	 * @param additionalInfo
	 *            used for logging
	 * @return the collection
	 * @throws RuntimeException
	 *             if the collection is empty
	 */
	public static Collection notEmpty(Collection collection, Object name,
			Object... additionalInfo) throws RuntimeException {
		Check.notNull(collection, name, additionalInfo);
		if (collection.isEmpty()) {
			StringBuilder result = new StringBuilder();
			result.append("Collection ").append(name).append(" is empty\n");
			// Check.appendAdditionalInfoAndThrowException(result,
			// additionalInfo);
			throw new IllegalArgumentException(result.toString());
		}
		return collection;
	}

	/**
	 * Get all successors, the successors of the successors, etc.
	 * 
	 * @param node
	 *            the inspected node
	 * @param result
	 *            will contain all nodes that can be reached from the node.
	 */
	public static void addReachableSuccessors(BoostedNode node,
			Set<BoostedNode> result) {
		for (BoostedNode successor : BoostedNodeUtils.getSuccessors(node)) {
			if (!result.contains(successor)) {
				result.add(successor);
				addReachableSuccessors(successor, result);
			}
		}
	}

	/**
	 * Computes all predecessors and their predecessors, etc..
	 * 
	 * @param node
	 *            the inspected node
	 * @param result
	 *            will contain all nodes from which the node can be reached.
	 */
	public static void addReachablePredecessors(BoostedNode node,
			Set<BoostedNode> result) {
		for (BoostedNode predecessor : BoostedNodeUtils
					.getPredecessors(node)) {
			if (!result.contains(predecessor)) {
				result.add(predecessor);
				addReachablePredecessors(predecessor, result);
			}
		}

	}

	/**
	 * Counts the number of outgoing edges.
	 * 
	 * @param node
	 *            the inspected node
	 * @return the number
	 */
	public static long getOutgoingEdgeCount(BoostedNode node) {
		long result = 0;
		for (BoostedNode outpoint : node.getOutPoints()) {
			result += node.getSuccessorInPoints(outpoint).size();
		}

		return result;
	}

	/**
	 * Counts the number of ingoing edges
	 * 
	 * @param node
	 *            the inspected node
	 * @return the number
	 */
	public static long getIngoingEdgeCount(BoostedNode node) {
		long result = 0;
		for (BoostedNode inpoint : node.getInPoints()) {
			result += node.getPredecessorOutPoints(inpoint).size();
		}

		return result;
	}

	/**
	 * Puts an element into set of a map
	 * 
	 * @param map
	 *            contains the key to set mapping
	 * @param key
	 *            the key of the value
	 * @param value
	 *            a new element for the set of the given key.
	 */
	public static <Key, Element> void putIntoMapOfSets(
			Map<Key, Set<Element>> map, Key key, Element value) {
		Set<Element> set = map.get(key);
		if (set == null) {
			set = new HashSet<Element>();
			map.put(key, set);
		}
		set.add(value);

	}

	/**
	 * Checks whether there may be a problem sorting the list. Problems may
	 * result because the comparison of objets may created ordering circles
	 * 
	 * @param comparator
	 *            the comparator used for sorting
	 * @param list
	 *            the inspected list
	 */
	@SuppressWarnings({ "rawtypes", "unchecked", "unused" })
	public static void findComparatorProblems(Comparator comparator, List list) {


		ArrayList list2 = new ArrayList(list);

		for (Object element1 : list) {
			for (Object element2 : list2) {
				int result1 = comparator.compare(element1, element2);
				int result2 = comparator.compare(element2, element1);
				if (result1 != -result2) {
					comparator.compare(element1, element2);
					comparator.compare(element2, element1);
					throw new IllegalStateException("Problems with " + element1
							+ "  " + element2);
				}

			}
		}

		Object problematicElement = null;
		while (problematicElement == null && !list2.isEmpty()) {
			problematicElement = list2.remove(0);

			try {
				Collections.sort(new ArrayList(list2), comparator);
				problematicElement = null;
			} catch (Exception ex) {
				//
				System.out.println("problematicElement stays != null");
			}
		}

		if (problematicElement != null) {
			list2.add(problematicElement);
			boolean changedList2 = true;

			while (changedList2) {
				changedList2 = false;
				for (Object element : new ArrayList(list2)) {
					if (problematicElement != element) {
						ArrayList list3 = new ArrayList(list2);
						list3.remove(element);
						try {
							Collections.sort(list3, comparator);
						} catch (Exception ex) {
							list2.remove(element);
							// because the error stays
							changedList2 = true;
						}

					}
				}
			}

			ArrayList list3 = new ArrayList(list2);
			list3.remove(problematicElement);
			Collections.sort(list3, comparator);

			int compareResult = -2;
			Object compareElement = null;
			for (Object element : list3) {
				int newCompare = comparator
						.compare(element, problematicElement);
				if (newCompare >= compareResult) {
					compareResult = newCompare;
					compareElement = element;
				} else {
					// for debugging breakpoints
					newCompare = comparator
							.compare(element, problematicElement);
					int compareBefore = comparator.compare(compareElement,
							problematicElement);
					int compareElements = comparator.compare(compareElement,
							element);
					throw new IllegalStateException(
							"ordering problem: problematic element "
									+ problematicElement
									+ " is less than element "
									+ element
									+ ", greater than "
									+ compareElement
									+ " but the compare element is greater than the element");
				}
			}

			throw new IllegalStateException("problematic element "
					+ problematicElement + "  in list " + list2);
		}

	}

	/**
	 * Gets one of the minimums. There may be more than one minimum, if the
	 * ordering is only partial
	 * 
	 * @param list
	 *            the inspected list
	 * @param comparator
	 *            compares the elements to find the minimum
	 * @return the found element of the list
	 */
	public static <Element> Element getMinimum(List<Element> list,
			Comparator<Element> comparator) {
		if (list.isEmpty())
			throw new IllegalStateException("no element in list");

		Element result = list.get(0);
		for (int i = list.size() - 1; i > 0; i--) {
			Element element = list.get(i);
			try {
				if (comparator.compare(element, result) < 0) {
					result = element;
				}
			} catch (IllegalStateException comparisonNotDefinedException) {
				// nothing to do
			}
		}

		return result;

	}

	/**
	 * Refactor into ContainerBoostUtils.
	 * 
	 * Sorts a list using a comparator.
	 * 
	 * @param list
	 *            the list to be modified
	 * @param comparator
	 *            there may be only a partial ordering
	 */
	public static <Element> void sortPartialOrderedList(List<Element> list,
			Comparator<Element> comparator) {
		try {
			Collections.sort(list, comparator);
		} catch (Exception orderingProblemException) {
			if (comparator instanceof BoostedComparator) {
				ArrayList<Element> result = new ArrayList<Element>(list.size());
				ArrayList<Element> changingList = new ArrayList<Element>(list);
				// try to get a partial ordering.
				while (!changingList.isEmpty()) {
					Element minimum = getMinimum(changingList, comparator);
					result.add(minimum);
					changingList.remove(minimum);
				}

				// check the ordering (easy check)
				for (int i = result.size() - 1; i > 0; i--) {
					Element element1 = result.get(i - 1);
					Element element2 = result.get(i);
					Integer comparison = ComparatorBoostUtils.compare(element1,
							element2, comparator);
					if (comparison != null && comparison.intValue() > 0) {
						ComparatorBoostUtils.compare(element1, element2,
								comparator);
						throw new IllegalStateException(
								"could not order list because " + element1
										+ " should not be greater than "
										+ element2);
					}
				}

				list.clear();
				list.addAll(result);

			} else {
				throw orderingProblemException;
			}
		}
	}

	/**
	 * Creates a simple boosted node with a given name.
	 * 
	 * @param name
	 *            the name of the node
	 * @return the created node
	 */
	public static <Name> SimpleBoostedNode<Name> createSimpleBoostedNode(
			Name name) throws Exception {
		SimpleBoostedNode<Name> result = new SimpleBoostedNode<Name>();
		result.setName(name);
		return result;
	}

	/**
	 * Refactor into BasicJavaCodeGenerator.
	 * 
	 * Creates the sourcecode of a setter invocation.
	 * 
	 * @param objectname
	 *            the object which method is invoked.
	 * @param setterName
	 *            the invoked setter
	 * @param propertyValue
	 *            the new value
	 * @param result
	 *            the sourcecode will be written into this string
	 * 
	 * @return 'objectname'.'setter'('propertyValue');
	 * @throws Exception
	 */
	public static String generateSetterInvocation(String objectname,
			String setterName, Codable propertyValue, StringBuilder result)
			throws Exception {
		String name;
		if (propertyValue == null) {
			name = null;
		} else {
			name = propertyValue.asSourcecode(result);
			TextBoostUtils.append(result, objectname, ".", setterName, "(",
					name, ");\n");
		}
		return name;
	}

	/**
	 * Creates an id that is unique for about 1 hour. The id is not too long so
	 * that it still is human readable in texts.
	 * 
	 * @return the created id
	 */
	public static String getTemporaryId() {
		ThreadBoostUtils.sleep(2);// so that the uid is unique
		String result = "" + System.currentTimeMillis();
		return StringUtils.right(result, 7);// so the number is not too big

	}

	/*
	 * public static void removeDuplicateEdges(BoostedNode node){
	 * HashSet<BoostedNode> successors = new HashSet<BoostedNode>(); for
	 * (BoostedNode out : node.getOutPoints()) { for (BoostedNode in :
	 * node.getSuccessorInPoints(out)) { BoostedNode succ =
	 * in.getZoomOut(null).getLeft(); if(successors.contains(succ)){
	 * Refactor.removeEdge(node, out, in, succ); } else { successors.add(succ);
	 * } } }
	 * 
	 * HashSet<BoostedNode> predecessors = new HashSet<BoostedNode>(); for
	 * (BoostedNode in : node.getInPoints()) { for (BoostedNode out :
	 * node.getPredecessorOutPoints(in)) { BoostedNode pre =
	 * out.getZoomOut(null).getLeft(); if (predecessors.contains(pre)){
	 * Refactor.removeEdge(pre, out, in, node); } else { predecessors.add(pre);
	 * } } }
	 * 
	 * }
	 */

	/**
	 * Checks that there are no duplicate edges from/to this node
	 * 
	 * @param node
	 *            the node to be inspected
	 */
	public static void checkNoDuplicateEdges(BoostedNode node) {
		HashSet<BoostedNode> successors = new HashSet<BoostedNode>();
		for (BoostedNode out : node.getOutPoints()) {
			for (BoostedNode in : node.getSuccessorInPoints(out)) {
				BoostedNode succ = in.getZoomOut(null).getLeft();
				Check.isTrue(!successors.contains(succ), "duplicate successor "
						+ succ);
				successors.add(succ);
			}
		}

		HashSet<BoostedNode> predecessors = new HashSet<BoostedNode>();
		for (BoostedNode in : node.getInPoints()) {
			for (BoostedNode out : node.getPredecessorOutPoints(in)) {
				BoostedNode pre = out.getZoomOut(null).getLeft();
				Check.isTrue(!predecessors.contains(pre),
						"duplicate predecessor " + pre);
				predecessors.add(pre);
			}
		}

	}

	/**
	 * Creates a frame with different parameters
	 * 
	 * @param title
	 *            the title of the frame
	 * @param width
	 *            the width of the frame
	 * @param height
	 *            the height of the frame
	 * @param disposeOnClosing
	 *            should the frame be disposed on closing
	 * @param onlyComponent
	 *            the central component of the frame
	 * @return the created frame
	 */
	public static JFrame createFrame(String title, int width, int height,
			boolean disposeOnClosing, Component onlyComponent) {
		SwingFactory guiFactory = new SwingFactory();
		JFrame frame = guiFactory.createFrame(title, width, height,
				disposeOnClosing, new BorderLayout());

		frame.getContentPane().add(new JScrollPane(onlyComponent),
				BorderLayout.CENTER);

		frame.setVisible(true);
		return frame;
	}

	// TODO add the feature into the transformation graph,
	// that the values are not computed beginning at the source nodes, but are
	// computed on request of the sinks

	/**
	 * Displays a graph of shapes.
	 * 
	 * @param configuration
	 *            the configuration of the graph and display
	 */
	public static <Edge> void displayShapes(
			DisplayConfiguration<Graph<? extends Shape, Edge>> configuration
	) {
		String title = configuration.getTitle();
		Point framePosition = configuration.getFramePosition();
		Rectangle preferredSize = configuration.getPreferredSize();
		// some data manipulation, because the content may be an Object[]
		ArrayList<Graph<Shape, Edge>> list = new ArrayList<Graph<Shape, Edge>>();
		for (Object content : configuration.getContent()) {
			list.add((Graph<Shape, Edge>) content);
		}

		Rectangle dim = GeometryBoostUtils.getGraphDimension((Collection) list);
		if (preferredSize == null) {
			preferredSize = dim;
		}
		if (framePosition == null) {
			framePosition = new Point(0, 0);
		}

		double xResizeFactor = 1;
		if (dim.width > preferredSize.width) {
			xResizeFactor = ((double) preferredSize.width) / dim.width;
		}

		double yResizeFactor = 1;
		if (dim.height > preferredSize.height) {
			yResizeFactor = ((double) preferredSize.height) / dim.height;
		}

		double xTranslate = 0;
		double minX = dim.x * xResizeFactor;
		if (minX != preferredSize.x) {
			xTranslate = preferredSize.x - minX;
		}

		double yTranslate = 0;
		double minY = dim.y * yResizeFactor;
		if (minY != preferredSize.y) {
			yTranslate = preferredSize.y - minY;
		}

		SwingFactory guiFactory = new SwingFactory();
		// add a margin to the dimension
		JFrame frame = guiFactory.createFrame(title, preferredSize.x
				+ preferredSize.width + 30, preferredSize.y
				+ preferredSize.height + 100,
				true, new BorderLayout());
		frame.setLocation(framePosition);
		Panel panel = new Panel();
		frame.getContentPane().add(panel, BorderLayout.CENTER);
		// frame.pack();
		// frame.setSize(400, 600);
		frame.setVisible(true);
		ThreadBoostUtils.sleep(3000);

		Color[] colors = { Color.BLACK, Color.RED, Color.BLUE, Color.CYAN,
				Color.GREEN, Color.MAGENTA, Color.ORANGE, Color.YELLOW };
		int index = 0;

		Graphics graphics = panel.getGraphics();

		for (Graph<? extends Shape, Edge> graph : list) {
			Color newColor = colors[index++ % colors.length];
			graphics.setColor(newColor);
			// panel.getGraphics().setPaintMode();

			for (Shape shape : graph.getVertices()) {
				Rectangle bounds = shape.getBounds();
				graphics.drawRect(
						scale(bounds.x, xResizeFactor, xTranslate),
						scale(bounds.y, yResizeFactor, yTranslate),
						scale(bounds.width, xResizeFactor, 0),
						scale(bounds.height, yResizeFactor, 0));
			}

			for (Edge edge : graph.getEdges()) {
				edu.uci.ics.jung.graph.util.Pair<? extends Shape> endpoints = graph
						.getEndpoints(edge);
				Rectangle point1 = endpoints.getFirst().getBounds();
				Rectangle point2 = endpoints.getSecond().getBounds();
				graphics.drawLine(
						scale(point1.x, xResizeFactor, xTranslate),
						scale(point1.y, yResizeFactor, yTranslate),
						scale(point2.x, xResizeFactor, xTranslate),
						scale(point2.y, yResizeFactor, yTranslate));

			}
		}
		ThreadBoostUtils.sleep(2000);
	}

	/**
	 * Scaling of a number with a given factor and translation
	 * 
	 * @param number
	 *            the number to change
	 * @param factor
	 *            the factor to scale
	 * @param translation
	 *            the translation
	 * @return number * factor + translation
	 */
	private static int scale(int number, double factor, double translation) {
		return (int) (number * factor + translation);
	}

	/**
	 * Creates an undirected graph with given vertices and edges
	 * 
	 * @param vertices
	 *            the vertices of the graph
	 * @param edges
	 *            the edges of the graph
	 * @return the created graph
	 */
	public static <Vertex> UndirectedSparseGraph<Vertex, String> createUndirectedGraph(
			Collection<Vertex> vertices, MultiMap<Vertex, Vertex> edges) {
		UndirectedSparseGraph<Vertex, String> result = new UndirectedSparseGraph<>();
		for (Vertex vertex : vertices) {
			result.addVertex(vertex);
		}

		int counter = 0;
		for (Entry<Vertex, Collection<Vertex>> edge : edges.entrySet()) {
			Vertex start = edge.getKey();
			for (Vertex end : edge.getValue()) {
				result.addEdge("edge" + counter++, start, end);
			}
		}

		return result;
	}

	/**
	 * Changes a (directed) graph into an undirected graph.
	 * 
	 * @param graph
	 *            the template
	 * @return the created graph
	 */
	public static <Vertex, Edge> UndirectedSparseGraph<Vertex, Edge> createUndirectedGraph(
			Graph<Vertex, Edge> graph) {
		UndirectedSparseGraph<Vertex, Edge> undirectedGraph = new UndirectedSparseGraph<Vertex, Edge>();
		for (Vertex vertex : graph.getVertices()) {
			undirectedGraph.addVertex(vertex);
		}

		for (Edge edge : graph.getEdges()) {
			undirectedGraph.addEdge(edge, graph.getEndpoints(edge));
		}

		return undirectedGraph;
	}

	/**
	 * Splits a graph into 2 subgraphs
	 * 
	 * @param graph
	 *            the inspected graph
	 * @param nodesOfFirstSubgraph
	 *            all these nodes and their connected components are put into
	 *            the first graph
	 * @return [graph with nodesOfFirstSubgraph, graph with other nodes, the
	 *         edges between graph1 and graph2 ]
	 */
	public static <Vertex, Edge> Triple<UndirectedGraph<Vertex, Edge>, UndirectedGraph<Vertex, Edge>, List<Triple<Edge, Vertex, Vertex>>> split(
			Graph<Vertex, Edge> graph, Collection<Vertex> nodesOfFirstSubgraph) {

		UndirectedSparseGraph<Vertex, Edge> graph1 = new UndirectedSparseGraph<Vertex, Edge>();
		UndirectedSparseGraph<Vertex, Edge> graph2 = new UndirectedSparseGraph<Vertex, Edge>();
		ArrayList<Triple<Edge, Vertex, Vertex>> edgesBetween = new ArrayList<Triple<Edge, Vertex, Vertex>>();

		for (Vertex vertex : graph.getVertices()) {
			if (nodesOfFirstSubgraph.contains(vertex)) {
				graph1.addVertex(vertex);
			} else {
				graph2.addVertex(vertex);
			}
		}

		for (Edge edge : graph.getEdges()) {
			edu.uci.ics.jung.graph.util.Pair<Vertex> endpoints = graph
					.getEndpoints(edge);
			Vertex point1 = endpoints.getFirst();
			Vertex point2 = endpoints.getSecond();

			boolean point1Graph1 = nodesOfFirstSubgraph.contains(point1);
			boolean point2Graph1 = nodesOfFirstSubgraph.contains(point2);

			if (point1Graph1 && point2Graph1) {
				graph1.addEdge(edge, endpoints);
			} else if ((!point1Graph1) && (!point2Graph1)) {
				graph2.addEdge(edge, endpoints);
			} else if (point1Graph1) {
				edgesBetween
.add(new Triple<Edge, Vertex, Vertex>(edge, point1,
						point2));
			} else { //
				edgesBetween
.add(new Triple<Edge, Vertex, Vertex>(edge, point2,
						point1));
			}

		}

		return new Triple<UndirectedGraph<Vertex, Edge>, UndirectedGraph<Vertex, Edge>, List<Triple<Edge, Vertex, Vertex>>>(
				graph1, graph2, edgesBetween);
	}

	/**
	 * Copies a graph.
	 * 
	 * @param graph
	 *            the original graph
	 * @return the graph with the Object-class as edges.
	 */
	public static <Vertex, Edge> UndirectedGraph<Vertex, Object> createGraphWithObjectEdges(
			UndirectedGraph<Vertex, Edge> graph) {
		UndirectedSparseGraph<Vertex, Object> result = new UndirectedSparseGraph<Vertex, Object>();
		for (Vertex vertex : graph.getVertices()) {
			result.addVertex(vertex);
		}

		for (Edge edge : graph.getEdges()) {
			result.addEdge(edge, graph.getEndpoints(edge));
		}

		return result;
	}

	/**
	 * Fetches one element of a collection.
	 * 
	 * @param collection
	 *            the inspected collection
	 * @return null if the collection is empty
	 */
	public static <Content> Content getOneOf(Collection<Content> collection) {
		if (collection.isEmpty())
			return null;
		else
			return collection.iterator().next();
	}

	/**
	 * Creates a panel with parts.
	 * 
	 * @param partsOfPanel
	 *            may be components or other objects (that are changed to
	 *            string)
	 * @return the created panel
	 */
	public static Panel createPanel(Object... partsOfPanel) {
		Panel result = new Panel();
		for (Object part : partsOfPanel) {
			if (part != null) {
				if (part instanceof Component) {
					result.add((Component) part);

				} else {
					result.add(new Label(part.toString()));
				}
			}
		}
		return result;
	}

	/**
	 * Finds a group of a pattern in a text
	 * 
	 * @param text
	 *            the text to be inspected
	 * @param pattern
	 *            the pattern to be evaluated
	 * @param groupNumber
	 *            the group of the pattern that is to be extracted.
	 * @return matched string + index of pattern
	 */
	public static Pair<String, Integer> findPattern(String text,
			String pattern,
			int groupNumber) {
		Pattern p = Pattern.compile(pattern);
		Matcher m = p.matcher(text);
		if (!m.find()) {
			throw new IllegalStateException("Could not find \n" + pattern
					+ "\nin \n" + text);
		}
		String result = m.group(groupNumber);
		return new ImmutablePair<String, Integer>(result, m.start());
	}
}
