package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.Rectangle;
import java.awt.Shape;
import java.util.Comparator;

/**
 * Comparator of shapes.
 * 
 * @author cbauer
 *
 */
public class ShapeComparator implements Comparator<Shape> {

	private static final int TYPE_FIRST_Y_THEN_X = 1;

	/**
	 * The type of this comparator
	 */
	private int type;

	/**
	 * Order according the x/y position of the bounds of the shape. Smallest y
	 * is lower, if y is equal then smaller x is lower
	 */
	public static final ShapeComparator YX_COMPARATOR = new ShapeComparator(
			TYPE_FIRST_Y_THEN_X);

	private ShapeComparator(int type) {
		this.type = type;
	}

	@Override
	public int compare(Shape shape1, Shape shape2) {

		Rectangle bounds1 = shape1.getBounds();
		int x1 = bounds1.x;
		int y1 = bounds1.y;

		Rectangle bounds2 = shape2.getBounds();
		int x2 = bounds2.x;
		int y2 = bounds2.y;

		switch (type) {
		case TYPE_FIRST_Y_THEN_X:
			if (y1 < y2) {
				return -1;
			} else if (y1 > y2) {
				return 1;
			} else {// equals
				if (x1 < x2) {
					return -1;
				} else if (x1 > x2) {
					return x2;
				} else {// equals
					return 0;
				}
			}
		default:
			throw new IllegalStateException("type not supported: " + type);
		}
	}


}
