package net.sf.gluebooster.demos.pojo.refactor;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import net.sf.gluebooster.java.booster.basic.gui.swing.SwingFactory;
import net.sf.gluebooster.java.booster.essentials.utils.IoBoostUtils;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

/**
 * Additional Methods
 * 
 * @author cbauer
 *
 */
public class SwingBoostUtils extends net.sf.gluebooster.java.booster.essentials.utils.SwingBoostUtils {

	/**
	 * Create a text field containing a file name together with a file chooser to select the file
	 * 
	 * @return the holder of the filename and the display component
	 */
	public static Pair<ComponentHolder, Component> createFileTextfieldWithFileChooser() throws Exception {
		JButton button = new JButton(">");
		JTextField textfield = new JTextField();
		ComponentHolder holder = new ComponentHolder(textfield);

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(textfield, BorderLayout.CENTER);
		panel.add(button, BorderLayout.EAST);
	
		DialogConfiguration dialog = new DialogConfiguration(null, "Select the file", "File Selection", JOptionPane.OK_CANCEL_OPTION);
		dialog.setReturnValueIfCancelled(true);

		TransformationGraph fieldValueSetter = TransformationGraph.createChain(//
				new TransformerToConstant<ComponentHolder>(holder), // "get the holder of the field //
				TransformerByInvoking.createMethodInvokingTransformer(null, "getValue"), // extract the field value
				TransformerByInvoking.createMethodInvokingTransformer(dialog, "setInitialValue"), // put the field value into the dialog as default
				new TransformerToConstant<DialogConfiguration>(dialog), // "get the dialog configuration//
				TransformerByInvoking.createMethodInvokingTransformer(new SwingUserInteraction(), "showFileDialog"), // and display the dialog
				TransformerByInvoking.createMethodInvokingTransformer(null, "getCanonicalPath"), // get the file path
				TransformerByInvoking.createMethodInvokingTransformer(holder, "setValue") // put it into the field
		);
		EventMulticaster listener = new EventMulticaster();
		listener.addCommandHandler(fieldValueSetter);
		button.setActionCommand(BasicCommands.COMMAND_START);
		button.addActionListener(listener);
	
		return new ImmutablePair<ComponentHolder, Component>(holder, panel);
	}

	/**
	 * Create a panel with multiple components.
	 * 
	 * @param components
	 *            the components of the panel.
	 * @return the created panel.
	 */
	public static JPanel createPanel(Component... components) {
		JPanel result = new JPanel();
		for (Component component : components) {
			result.add(component);
		}
		return result;
	}

	/**
	 * Creates a textframe that reads its content from a file
	 * 
	 * @param file
	 *            contains the text
	 * @param encoding
	 *            encoding of the file
	 * @param width
	 *            width of the frame
	 * @param height
	 *            height of the frame
	 * @return the filled in frame
	 */
	public static JFrame createFiletextFrame(File file, String encoding, int width, int height) throws Exception {
		String text;
		if (encoding == null) {
			text = FileUtils.readFileToString(file);
		} else {
			text = FileUtils.readFileToString(file, encoding);
		}
		return createTextFrame(file.getCanonicalPath(), text, width, height);
	}

	/**
	 * Creates textfile and a textframe that reads its content from the file
	 * 
	 * @param textOfFile
	 *            the text to write
	 * @param file
	 *            the file to write
	 * @param width
	 *            width of the frame
	 * @param height
	 *            height of the frame
	 * @return the filled in frame
	 */
	public static JFrame createFiletextFrame(CharSequence textOfFile, File file, int width, int height) throws Exception {
		String encoding = IoBoostUtils.UTF_8;
		FileUtils.write(file, textOfFile, encoding);
		return createFiletextFrame(file, encoding, width, height);
	}

	/**
	 * Create a frame with a textarea.
	 * 
	 * @param title
	 *            the title of the frame
	 * @param text
	 *            the text of the frame
	 * @param width
	 *            width of the frame
	 * @param height
	 *            height of the frame
	 * @return the filled in frame
	 */
	public static JFrame createTextFrame(String title, Object text, int width, int height) {
		JFrame frame = new SwingFactory().createFrame(title, width, height, true, new BorderLayout());
		JTextArea area = new JTextArea();
		area.setText("" + text);
		frame.getContentPane().add(new JScrollPane(area), BorderLayout.CENTER);
		return frame;
	}

}
