package net.sf.gluebooster.demos.pojo.refactor;


/**
 * Selects an element of a list/map.
 * 
 * @author CBauer
 *
 * @param <Key>
 * @param <Value>
 */
public class TransformerByForEach extends BoostedObjectFactory<Object, Object> {

	/**
	 * Should the transformation be done for each and a list returned.
	 */
	private static final int TYPE_RETURN_LIST = 1;

	/**
	 * Should an iterator be returned that for each invocation does the
	 * transformation of one element.
	 */
	private static final int TYPE_RETURN_ITERATOR = 2;

	/**
	 * The type of this transformer
	 */
	private int type;

	/**
	 * The transformer of the elements
	 */
	private TransformingDelegate<?, ?> elementTransformer;

	/**
	 * Creates a transformer that invokes the element transformer on each
	 * element and returns a list.
	 * 
	 * @param elementTransformer
	 *            the transformer of the elements
	 * @return the created transformer
	 */
	public static TransformerByForEach createReturnListTransformer(
			TransformingDelegate<?, ?> elementTransformer) {
		return new TransformerByForEach(TYPE_RETURN_LIST, elementTransformer);
	}

	private TransformerByForEach(int type,
			TransformingDelegate<?, ?> elementTransformer) {
		this.type = type;
		this.elementTransformer = elementTransformer;
	}

	@Override
	protected Object transformWithException(Object input) {

		switch (type) {
		case TYPE_RETURN_LIST:
		case TYPE_RETURN_ITERATOR:
		default:
			throw new IllegalStateException("type not supported " + type);
		}
	}


}
