package net.sf.gluebooster.demos.pojo.refactor;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Transforms by using a regular expression
 * 
 * @author CBauer
 *
 * @param <Key>
 * @param <Value>
 */
public class TransformerByRegex extends BoostedObjectFactory<String, Object> {

	private static final int TYPE_RETURN_FIRST_MATCH = 1;

	/**
	 * The type of this transformer
	 */
	private int type;

	/**
	 * The pattern to be used.
	 */
	private Pattern pattern;

	/**
	 * Creates a transformer that returns the first match of the pattern.
	 * 
	 * @param pattern
	 *            the pattern that is to be matched
	 * @return the created transformer
	 */
	public static TransformerByRegex createReturnFirstMatchTransformer(
			String pattern) {
		return new TransformerByRegex(TYPE_RETURN_FIRST_MATCH,
				Pattern.compile(pattern));
	}

	private TransformerByRegex(int type, Pattern pattern) {
		this.type = type;
		this.pattern = pattern;
	}

	@Override
	protected Object transformWithException(String input) {

		switch (type) {
		case TYPE_RETURN_FIRST_MATCH:
			Matcher matcher = pattern.matcher(input);
			if (!matcher.find()) {
				throw new IllegalStateException("no match found for pattern '"
						+ pattern.pattern() + "' in string '" + input + "'");
			} else {
				String result = input.substring(matcher.start(), matcher.end());
				return result;
			}

		default:
			throw new IllegalStateException("type not supported " + type);
		}
	}


}
