package net.sf.gluebooster.demos.pojo.refactor;

import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.tuple.Pair;

/**
 * Selects an element of a list/map.
 * 
 * @author CBauer
 *
 * @param <Key>
 * @param <Value>
 */
public class TransformerBySelectingAnElement<Value> extends
		BoostedObjectFactory<Object, Value> {

	/**
	 * Must be an integer (the index) if the input is a list or pair (0 = Left,
	 * 1 = Right). If the input is a map it is the key
	 */
	private Object selector;

	public TransformerBySelectingAnElement(Object selector) {
		this.selector = selector;
		setName(TransformerBySelectingAnElement.class.getSimpleName() + ":"
				+ selector);
	}


	@Override
	protected Value transformWithException(Object input) {

		if (input == null) {
			throw new IllegalStateException("Input must not be null ");
		} else if (input instanceof List<?>) {
			return ((List<Value>) input).get(getIntSelector());
		} else if (input instanceof Pair<?, ?>) {
			switch (getIntSelector()) {
			case 0:
				return ((Pair<Value, ?>) input).getLeft();
			case 1:
				return ((Pair<?, Value>) input).getRight();
			default:
				throw new IllegalStateException(
						"Selector not supported for pair: " + selector);
			}
		} else if (input instanceof Map<?, ?>) {
			return ((Map<?, Value>) input).get(selector);
		} else {
			throw new IllegalStateException("Selection not yet supported "
					+ input.getClass().getName());
		}

	}

	/**
	 * Returns the selector as int value
	 * @return selector
	 */
	private int getIntSelector() {
		return ((Number) selector).intValue();
	}

}
