package net.sf.gluebooster.demos.pojo.refactor;

import java.util.Collection;
import java.util.Iterator;

/**
 * Transforms an object into an instance of a given class.
 * A type can determine the concrete transformation algorithm.
 * @author cbauer
 *
 * @param <Target> the class of which instances are returned
 */
public class TransformerToClass<Target> extends
		BoostedObjectFactory<Object, Target> {

	/**
	 * Transforms into an iterator
	 */
	private static final int TYPE_ITERATOR = 1;

	/**
	 * The type of the transformation 
	 */
	private int type;

	/**
	 * Creates an transformer that returns iterators.
	 * 
	 * @return the created transformer
	 */
	public static BoostedObjectFactory<Object, Iterator> createToIteratorTransformer() {
		return new TransformerToClass<Iterator>(TYPE_ITERATOR);
	}

	private TransformerToClass(int type) {
		this.type = type;
	}

	@Override
	protected Target transformWithException(Object input) throws Exception {
		switch (type) {
		case TYPE_ITERATOR:
			if (input instanceof Collection) {
				return (Target) ((Collection) input).iterator();
			} else {
				throw new IllegalStateException(
						"input not supported for type iterator: " + input);
			}
		default:
			throw new IllegalStateException("type not supported " + type);
		}
	}


}
