package net.sf.gluebooster.demos.pojo.refactor;

import javax.naming.spi.ObjectFactory;

import net.sf.gluebooster.java.booster.essentials.utils.ReflectionBoostUtils;

/**
 * Always returns a constant value.
 * 
 * @author CBauer
 *
 */
public class TransformerToConstant<ConstantClass> extends
		BoostedObjectFactory<Object, ConstantClass> {

	/**
	 * The result that is to be returned.
	 */
	private ConstantClass value;
	
	/**
	 * Is a cloned value to be returned.
	 */
	private boolean cloneTheOriginal = false;

	/**
	 * Creates the value if it is null;
	 */
	private ObjectFactory initializer;

	public TransformerToConstant() {
	}

	public TransformerToConstant(ConstantClass value) {
		this.value = value;
	}

	/**
	 * 
	 * @param name
	 *            the name of this tranformer
	 */
	public TransformerToConstant(ConstantClass value, Object name) {
		super(name);
		this.value = value;
	}

	public TransformerToConstant(ConstantClass value, boolean cloneTheOriginal) {
		this.value = value;
		this.cloneTheOriginal = cloneTheOriginal;
	}

	/**
	 * Creates the constant factory. The value is initalized with the first access
	 * 
	 * @param initializer
	 *            factory to create the value
	 * @return the constant factory
	 */
	public static <ConstantClass> TransformerToConstant<ConstantClass> createWithInitializer(ObjectFactory initializer) {
		TransformerToConstant<ConstantClass> result = new TransformerToConstant<ConstantClass>();
		result.setInitializer(initializer);
		return result;
	}

	public ConstantClass getValue() throws Exception {
		return transformWithException(null);
	}

	public void setValue(ConstantClass value) {
		this.value = value;
	}

	@Override
	protected ConstantClass transformWithException(Object ignored)
			throws Exception {

		if (value == null && initializer != null) {
			value = (ConstantClass) initializer.getObjectInstance(null, null, null, null);
		}

		if (cloneTheOriginal) {
			return (ConstantClass) ReflectionBoostUtils.getMethod(
					value.getClass(), "clone").invoke(value);
		} else {
			return value;
		}
	}

	public boolean isCloneTheOriginal() {
		return cloneTheOriginal;
	}

	public void setCloneTheOriginal(boolean cloneTheOriginal) {
		this.cloneTheOriginal = cloneTheOriginal;
	}

	public ObjectFactory getInitializer() {
		return initializer;
	}

	public void setInitializer(ObjectFactory initializer) {
		this.initializer = initializer;
	}

}
