package net.sf.gluebooster.demos.pojo.refactor;

import java.io.InputStream;

/**
 * Creates an input stream.
 * The concrete transformation algorithm is determined by the type of a transformer.
 * 
 * @author CBauer
 *
 */
public class TransformerToInputStream extends
		BoostedObjectFactory<Object, InputStream> {

	/**
	 * A classpath resource is to be read.
	 */
	private static final int TYPE_CLASSPATH_RESSOURCE = 1;

	/**
	 * The type of the transformer. 
	 */
	private int type;

	/**
	 * The class that is used.
	 */
	private Class clasz;
	
	/**
	 * The path that is used.
	 */
	private String path;

	/**
	 * Creates a transformer that returns the content of a given classpath resource
	 * @param clasz the class that is used to get the resource
	 * @param pathRelativeToClasz the path to the resource relative to the clasz
	 * @return the created transformer
	 */
	public static TransformerToInputStream createClasspathTransformer(
			Class clasz, String pathRelativeToClasz) {
		TransformerToInputStream result = new TransformerToInputStream(
				TYPE_CLASSPATH_RESSOURCE, pathRelativeToClasz);
		result.clasz = clasz;
		return result;
	}

	/**
	 * Constructor with type and path.
	 */
	private TransformerToInputStream(int type, String path) {
		this.type = type;
		this.path = path;
	}

	@Override
	protected InputStream transformWithException(Object input) throws Exception {
		switch (type) {
		case TYPE_CLASSPATH_RESSOURCE:
			return clasz.getResourceAsStream(path);
		default:
			throw new IllegalStateException("Type not supported: " + type);

		}
	}


}
