package net.sf.gluebooster.demos.pojo.refactor;

import java.io.InputStream;
import java.io.Reader;

import org.apache.commons.io.IOUtils;

/**
 * Transforms object to a string. The objects can be streams, etc.
 * The transformation algorithm is determined by the type of a transformer.
 * 
 * @author CBauer
 *
 */
public class TransformerToString extends BoostedObjectFactory<Object, String> {

	/**
	 * Read all of a stream, etc.
	 */
	private static final int TYPE_READ_ALL = 1;

	/**
	 * The type of the transformer.
	 */
	private int type;

	/**
	 * Creates a transformer that reads all from a stream, reader, ...
	 * @return the created transformer
	 */
	public static TransformerToString createReadingAllTransformer() {
		TransformerToString result = new TransformerToString(TYPE_READ_ALL);
		return result;
	}

	private TransformerToString(int type) {
		this.type = type;
	}

	/**
	 * Currently supported input classes are InputStream, Reader, byte[]. With
	 * all other objects the toString method is invoked.
	 */
	@Override
	protected String transformWithException(Object input) throws Exception {
		if (input == null) {
			return null;
		}

		switch (type) {
		case TYPE_READ_ALL:
			if (input instanceof InputStream) {
				return IOUtils.toString((InputStream) input);
			} else if (input instanceof Reader) {
				return IOUtils.toString((Reader) input);
			} else if (input instanceof byte[]) {
				return IOUtils.toString((byte[]) input);
			} else {
				return input.toString();
			}

		default:
			throw new IllegalStateException("Type not supported: " + type);

		}
	}


}
