package net.sf.gluebooster.demos.pojo.refactor;

import javax.naming.spi.ObjectFactory;

import net.sf.gluebooster.java.booster.essentials.meta.HasName;
import net.sf.gluebooster.java.booster.essentials.utils.Check;

import org.apache.commons.collections15.Transformer;

/**
 * Transforms objects by delegating to a factory.
 * 
 * @author CBauer
 *
 */
public class TransformingDelegate<Source, Result> extends
		BoostedObjectFactory<Source, Result> {

	/**
	 * The supported classes that the factory may have..
	 */
	private static final Class[] supportedClasses = { ObjectFactory.class,
			Transformer.class, org.apache.commons.collections.Transformer.class };

	/**
	 * The factory that does the transforming.
	 */
	private Object factory;

	public TransformingDelegate(Object factory) {
		setFactory(factory);
	}


	public void setFactory(Object factory) {
		Check.notNull(factory, "factory");

		boolean checkSuccessful = false;
		for (Class checkClass : supportedClasses) {
			if (checkClass.isInstance(factory)) {
				checkSuccessful = true;
				break;
			}
		}

		if (!checkSuccessful) {
			throw new IllegalArgumentException(
					"factory class not (yet) supported " + factory.getClass());
		}

		this.factory = factory;

		if (factory instanceof HasName) {
			Object name = ((HasName) factory).getName();
			if (name != null)
				setName(name);
			else
				setName(factory.getClass().getSimpleName());

		}

	}


	@Override
	protected Result transformWithException(Source input) throws Exception {
		if (factory instanceof ObjectFactory) {
			return (Result) ((ObjectFactory) factory).getObjectInstance(input,
					null, null, null);
		} else if (factory instanceof Transformer) {
			return (Result) ((Transformer) factory).transform(input);
		} else if (factory instanceof org.apache.commons.collections.Transformer) {
			return (Result) ((org.apache.commons.collections.Transformer) factory)
					.transform(input);
		} else {
			throw new IllegalStateException("factory not supported " + factory);
		}

	}

}
