package net.sf.gluebooster.demos.pojo.refactor.gb;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import net.sf.gluebooster.demos.pojo.refactor.AppControllerAbstraction;
import net.sf.gluebooster.demos.pojo.refactor.AppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.ConfigurationBoostUtils;
import net.sf.gluebooster.demos.pojo.refactor.ContainerBoostUtils;

/**
 * Implementation of the controller with some features common to the apps of this module.
 * 
 * @author cbauer
 *
 */
public abstract class AppControllerImpl extends AppControllerAbstraction<String> {

	/**
	 * The definition of the application.
	 */
	private AppDefinition appDefinition;

	/**
	 * 
	 * @param definition
	 *            The definition of the application.
	 */
	protected AppControllerImpl(AppDefinition definition) {
		appDefinition = definition;
	}

	/**
	 * If the parameter is an object array with only 1 element, it returns the element. Otherwise the parameter is returned.
	 * 
	 * @param parameter
	 *            will be inspected
	 * @return parameter or parameter[0]
	 */
	protected <Result> Result getSingleParameter(Object parameter) {
		if (parameter != null && parameter.getClass().isArray() && Array.getLength(parameter) == 1) {
			return (Result) Array.get(parameter, 0);
		} else {
			return (Result) parameter;
		}
	}

	@Override
	protected boolean executeCommandImpl(String commandName, Object parameter) throws Exception {
		if (AppDefinition.COMMAND_SAVE_CONFIGURATION.equals(commandName)) {
			File file = (File) parameter;
			ConfigurationBoostUtils.saveConfiguration(getModel(), appDefinition.getConfigurationFields(), new FileOutputStream(file), true);
			return true;
		} else if (AppDefinition.COMMAND_LOAD_CONFIGURATION.equals(commandName)) {
			File file = getSingleParameter(parameter);

			Map<Object, Object> configurationFields = new HashMap<>();

			ContainerBoostUtils.copy((Map) ConfigurationBoostUtils.loadConfiguration(new FileInputStream(file), true), configurationFields,
					appDefinition.getConfigurationFields(), true);
			executeCommand(AppDefinition.COMMAND_EDITED_CONFIGURATION, configurationFields);
			// the reaction to the changed configuration (maybe only copying into the model) must be done by sublasses (when necessary)

			return true;
		} else if (AppDefinition.COMMAND_EDITED_CONFIGURATION.equals(commandName)) {
			// the fields are copied into the model
			Map<Object, Object> configurationFields = getSingleParameter(parameter);
			ContainerBoostUtils.copy(configurationFields, getModel(), configurationFields.keySet(), true);
			return false; // subclasses may need to interpret the loaded configuration

		}
		return false;
	}

	// TODO load initial configuration from file if possible when starting (command: appdefinition.start)
}
