package net.sf.gluebooster.demos.pojo.refactor.gb;

import java.util.Collection;

import org.apache.commons.configuration.XMLConfiguration;

import net.sf.gluebooster.demos.pojo.languages.chinese.ChineseAppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.AppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.AppViewAbstraction;
import net.sf.gluebooster.demos.pojo.refactor.CommandHandler;
import net.sf.gluebooster.demos.pojo.refactor.DialogConfiguration;

/**
 * Additional view methods for the demo applications.
 * 
 * @author cbauer
 *
 * @param <Controller>
 */
public abstract class AppViewImpl<Controller extends CommandHandler, Definition extends AppDefinition> extends AppViewAbstraction<Controller, Definition> {


	/**
	 * @param appDefinition
	 *            the definition of the application
	 */
	public AppViewImpl(Definition appDefinition) {
		super(appDefinition);
	}

	/**
	 * Executes the default commands. Other commands can be implemen Because it throws an exception if the command is
	 * 
	 * @throws Exception
	 *             if the command is not supported.
	 */
	@Override
	public final Object executeCommand(Object commandName, Object parameter) throws Exception {


		if (execute(commandName, parameter)) {
			// everything ok
		} else if (AppDefinition.COMMAND_LOAD_CONFIGURATION.equals(commandName)) {

			DialogConfiguration dialog = new DialogConfiguration("Load configuration from", null);
			dialog.setDialogType(DialogConfiguration.TYPE_CHOOSE_FILE);

			controllerCommandWithFile(ChineseAppDefinition.COMMAND_LOAD_CONFIGURATION, dialog);

		} else if (AppDefinition.COMMAND_SAVE_CONFIGURATION.equals(commandName)) {

			DialogConfiguration dialog = new DialogConfiguration("Save configuration to", null);
			dialog.setDialogType(DialogConfiguration.TYPE_CHOOSE_FILE);

			controllerCommandWithFile(AppDefinition.COMMAND_SAVE_CONFIGURATION, dialog);

		} else if (AppDefinition.COMMAND_EDIT_CONFIGURATION.equals(commandName)) {
			Collection<Object> fields = getAppDefinition().getConfigurationFields();
			copyFromControllerToDisplayModel(fields);
			getDisplayModel().put(AppDefinition.DIALOG_DISPLAY_CONFIGURATION, Boolean.TRUE);// display
			// this part may be invoked directly or only after the closing of the dialog. Depends on the display model
		} else if (AppDefinition.COMMAND_EDITED_CONFIGURATION.equals(commandName)) {
			controllerCommandWithFields(AppDefinition.COMMAND_EDITED_CONFIGURATION, getAppDefinition().getConfigurationFields());
		} else {
			throw new IllegalStateException("Unsupported command: " + commandName);
		}
		return null;
	}

	/**
	 * Execute commands. Default commands that are already handled in the Must be overwritten if other commands are to be executed
	 * 
	 * @param commandName
	 *            the name of the command
	 * @param parameter
	 *            parameter of the command
	 * @return true if the command has been executed
	 */
	protected boolean execute(Object commandName, Object parameter) throws Exception {
		return false;
	}


}
