package net.sf.gluebooster.demos.pojo.languages.chinese;

import java.io.File;
import java.util.Map;

import javax.swing.JFrame;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import net.sf.gluebooster.demos.pojo.refactor.AppDefinition;
import net.sf.gluebooster.demos.pojo.refactor.AppDisplayBySwingBorderLayout;
import net.sf.gluebooster.demos.pojo.refactor.AppViewAbstraction;
import net.sf.gluebooster.demos.pojo.refactor.CommandHandler;
import net.sf.gluebooster.demos.pojo.refactor.ExtendedTestRoot;
import net.sf.gluebooster.demos.pojo.refactor.SwingBoostUtils;
import net.sf.gluebooster.java.booster.essentials.meta.Example;

public class ChineseDemoTest extends ExtendedTestRoot implements CommandHandler<String, Object, Object> {

	private static final String CLOSE_CONFIGURATION = "close configuration";

	private AppDisplayBySwingBorderLayout demoLayout;

	public ChineseDemoTest() {
	}


	/**
	 * An example of the features of the chinese app.
	 * 
	 * 
	 * @throws Exception
	 */
	@Test
	@Example(clasz = ChineseDemo.class)
	public void chineseDemo() throws Exception {
		final File directory = createTempDirectory("images");
		File tempDirectory = createTempDirectory("temp");

		// [gb:useInDocumentation]
		// Initialization
		AppDisplayBySwingBorderLayout layout = ChineseDemo.startSampleApp();
		final JFrame frame = layout.getFrame();
		createScreenshot(directory, "step1", frame);
		// </pre>Initial screen: <br/><img src="images/step1.png"/><pre>
		// [/gb:useInDocumentation]
		final AppViewAbstraction view = layout.getViewLogic();

		ChineseDemoTest commandHandler = new ChineseDemoTest();
		commandHandler.demoLayout = layout;

		createScreenshotInThread(directory, "step1a", frame, commandHandler, AppDefinition.COMMAND_EDIT_CONFIGURATION, CLOSE_CONFIGURATION);
		// [gb:useInDocumentation]
		// </pre>The original empty configuration <br/><img src="images/step1a.png"/><pre>
		// [/gb:useInDocumentation]

		StringBuilder dictionarytext = new StringBuilder();
		dictionarytext.append("麼 么 [me5] /(Fragepartikel der Umgangssprache) (u.E.)/\r\n");
		dictionarytext.append(
				"麻 麻 [ma2] /Anästhesie (u.E.) (S, Med)/beschäftigen, ärgern (u.E.) (S)/Hanf (u.E.) (S)/prickeln, taub werden (u.E.) (V)/betäubt, empfindungslos (u.E.) (Adj)/uneben, rauh, pockenartig (u.E.) (Adj)/Radikal Nr. 200 = Hanf, Flachs, Jute, Sesam (u.E.) (S)/\r\n");
		dictionarytext.append(
				"廣 广 [guang3] /breit, umfangreich, weit reichend (u.E.)/weit (u.E.)/Radikal Nr. 53 = Schrägdach, Schutz, Unterschlupf, Unterbringung (u.E.) (S)/\r\n");
		dictionarytext.append("林 林 [lin2] /Wald (u.E.) (S)/Lin (u.E.) (Eig, Fam)/\r\n");
		dictionarytext.append(
				"木 木 [mu4] /Baum (S)/Holz (S)/Nutzholz (S)/hölzern, aus Holz gemacht (Adj)/Radikal Nr. 75 = Baum, Holz/erstarrt, empfindungslos, taub (Adj)/\r\n");
		dictionarytext
				.append("么 幺 [yao1] /eins ( Aussprache der Ziffer 1 ) (u.E.) (Adj)/jünste (u.E.) (Adj)/klein, fein, winzig, unbedeutend (u.E.) (Adj)/\r\n");
		dictionarytext.append(
				"幺 幺 [yao1] /eins (Aussprache der Ziffer 1) (u.E.)/Radikal Nr. 52 = jung, gering, klein, unreif, jüngst; früher auch: zart, fein (u.E.)/\r\n");

		File dictionaryFile = writeTextfileAndCreateScreenshot(new File(tempDirectory, "dictionary.txt"), dictionarytext, 900, 480, directory, "step2");
		// [gb:useInDocumentation]
		// </pre>A simple small dictionary <br/><img src="images/step2.png"/><pre>
		// [/gb:useInDocumentation]

		StringBuilder characterComponents = new StringBuilder();
		characterComponents.append("<pre>\r\n");
		characterComponents.append("	麼	14	吕	麻	11		幺	3		ICVI	麻\r\n");
		characterComponents.append("	麻	11	吕	广	3		林	8		IJCC	*\r\n");
		characterComponents.append("	林	8	吅	木	4		*	4		DD	木\r\n");
		characterComponents.append("</pre>\r\n");
		characterComponents.append("\r\n");

		File componentsFile = writeTextfileAndCreateScreenshot(new File(tempDirectory, "components.txt"), characterComponents, 1024, 480, directory, "step3");
		// [gb:useInDocumentation]
		// </pre>A simple small components files <br/><img src="images/step3.png"/><pre>
		// [/gb:useInDocumentation]

		File defaultResult = new File(tempDirectory, "result.txt");

		Map<Object, Object> controllerModel = (Map<Object, Object>) view.getController().executeCommand(AppDefinition.COMMAND_GET_MODEL, null);

		controllerModel.put(ChineseAppDefinition.CONFIGURATION_FIELD_CHARACTER_COMPONENTS_FILE, componentsFile.getCanonicalPath());
		controllerModel.put(ChineseAppDefinition.CONFIGURATION_FIELD_CHINESE_LANGUAGE1_FILE, dictionaryFile.getCanonicalPath());
		controllerModel.put(ChineseAppDefinition.CONFIGURATION_FIELD_DEFAULT_RESULT_FILE, defaultResult.getCanonicalPath());

		createScreenshotInThread(directory, "step4", frame, commandHandler, AppDefinition.COMMAND_EDIT_CONFIGURATION, CLOSE_CONFIGURATION);
		// [gb:useInDocumentation]
		// </pre>The configuration with the given files <br/><img src="images/step4.png"/><pre>
		// [/gb:useInDocumentation]

		view.getDisplayModel().put(ChineseAppDefinition.FIELD_ORIGINAL_TEXT, "麼廣木木木木木木木木木木木木木木");
		createScreenshot(directory, "step5", frame);
		// [gb:useInDocumentation]
		// </pre>Insert some traditional characters <br/><img src="images/step5.png"/><pre>
		// [/gb:useInDocumentation]

		view.executeCommand(ChineseAppDefinition.COMMAND_CHANGE_TRADITIONAL_TO_SIMPLIFIED, null);
		createScreenshot(directory, "step6", frame);
		// [gb:useInDocumentation]
		// </pre>Click the button the convert from traditional to simplified characters and you see the conversion <br/><img src="images/step6.png"/><pre>
		// [/gb:useInDocumentation]

		view.executeCommand(ChineseAppDefinition.COMMAND_CREATE_VOCABULARY, null);
		createScreenshot(directory, "step7", frame);
		// [gb:useInDocumentation]
		// </pre>Click the button to create the vocabulary of the text <br/><img src="images/step7.png"/><pre>
		// [/gb:useInDocumentation]

		view.executeCommand(ChineseAppDefinition.COMMAND_EXTRACT_CHARACTERS, null);
		createScreenshot(directory, "step8", frame);
		// [gb:useInDocumentation]
		// </pre>Click the button to extract the vocabulary of the text <br/><img src="images/step8.png"/><pre>
		// [/gb:useInDocumentation]

		view.getDisplayModel().put(ChineseAppDefinition.FIELD_ORIGINAL_TEXT, "麼\r\n廣\r\n木");
		view.getDisplayModel().put(ChineseAppDefinition.FIELD_TRANSFORMED_TEXT, "麼廣\r\n麼木\r\n木木");
		createScreenshot(directory, "step9", frame);
		// [gb:useInDocumentation]
		// </pre>Insert the vocabulary into the left textarea, the text to be sorted into the vocabulary in the right textarea. <br/><img
		// src="images/step9.png"/><pre>
		// [/gb:useInDocumentation]

		view.executeCommand(ChineseAppDefinition.COMMAND_SORT_TEXT, null);
		createScreenshot(directory, "step10", frame);
		// [gb:useInDocumentation]
		// </pre>Click the button sort the right text into the vocabulary <br/><img src="images/step10.png"/><pre>
		// [/gb:useInDocumentation]

		copyExampleResultFiles(directory);
		// frame.setVisible(false);
		frame.dispose();
	}

	@Override
	public Object executeCommand(String commandName, Object parameter) throws Exception {
		AppViewAbstraction view = demoLayout.getViewLogic();

		switch (commandName) {
		case CLOSE_CONFIGURATION:
			view.executeCommand(AppDefinition.COMMAND_EDITED_CONFIGURATION, null);
			view.getDisplayModel().put(AppDefinition.DIALOG_DISPLAY_CONFIGURATION, Boolean.FALSE);
			break;
		case AppDefinition.COMMAND_EDIT_CONFIGURATION:
			view.executeCommand(AppDefinition.COMMAND_EDIT_CONFIGURATION, null);
			break;
		}

		return null;
	}

}
