package net.sf.gluebooster.demos.pojo.refactor;

import java.util.Set;

import junit.framework.Assert;

import org.junit.Test;

import edu.uci.ics.jung.graph.UndirectedGraph;
import edu.uci.ics.jung.graph.UndirectedSparseGraph;

public class GraphSequenceClustererTest {

	@Test
	public void testTransform() throws Exception {

		UndirectedGraph<String, String> graph = new UndirectedSparseGraph<String, String>();
		GraphSequenceClusterer<String, String> sequencer = new GraphSequenceClusterer<String, String>();

		for (int i = 0; i < 12; i++) {
			graph.addVertex(vertex( i));
		}


		Set<Set<String>> sequences = sequencer.transform(graph);
		Assert.assertTrue("no sequences because no edges ", sequences.isEmpty());

		graph.addEdge("0-1", "0", "1");
		sequences = sequencer.transform(graph);
		Assert.assertFalse("one sequences 0-1", sequences.isEmpty());

		graph.addEdge("1-2", "1", "2");
		graph.addEdge("2-3", "2", "3");

		sequences = sequencer.transform(graph);
		Assert.assertEquals("one sequences 0-1-2-3", 1, sequences.size());
		Assert.assertEquals(4, sequences.iterator().next().size());

		graph.addEdge("4-5", "4", "5");
		graph.addEdge("5-6", "5", "6");
		graph.addEdge("6-7", "6", "7");
		sequences = sequencer.transform(graph);
		Assert.assertEquals("two sequences 0-1-2-3,  4-5-6-7", 2,
				sequences.size());
		// both sequences have size 4
		for (Set<String> sequence : sequences) {
			Assert.assertEquals(4, sequence.size());
		}

		graph.addEdge("3-8", "3", "8");
		graph.addEdge("3-9", "3", "9");
		sequences = sequencer.transform(graph);
		Assert.assertTrue("there should be at least 2 seqences ",
				sequences.size() >= 2);

		graph.addEdge("11-0", "11", "0");
		graph.addEdge("12-0", "12", "0");
		sequences = sequencer.transform(graph);
		Assert.assertTrue("there should be at least 2 seqences ",
				sequences.size() >= 2);

		// add a circle
		graph.addEdge("3-0", "3", "0");
		sequences = sequencer.transform(graph);
		Assert.assertTrue("there should be at least 2 seqences ",
				sequences.size() >= 2);
		for (Set<String> sequence : sequences) {
			Assert.assertFalse("there should be no seqeunce 0-1-2-3 any more",
					sequence.contains("0") && sequence.contains("1")
							&& sequence.contains("2") && sequence.contains("3"));
		}
		

	}

	private String vertex(int i){
		return "" + i;
	}
	
	@Test
	public void test2Transform() throws Exception {

		UndirectedGraph<String, String> graph = new UndirectedSparseGraph<String, String>();
		//create sequence
		int numberOfNodes = 4;
		for (int i = 0; i < numberOfNodes; i++) {
		    String pre = null;
		    if (i > 0){
		    	pre = vertex(i - 1);
		    }
		    String current = vertex(i);
		
			graph.addVertex(current);
			if (pre != null){
				graph.addEdge( pre + "-" + current, pre, current);
			}
		}

		GraphSequenceClusterer<String, String> sequencer = new GraphSequenceClusterer<String, String>();
		Set<Set<String>> sequences = sequencer.transform(graph);
		Assert.assertEquals(numberOfNodes, sequences.iterator().next().size());
	}
}
