package net.sf.gluebooster.demos.pojo.refactor;

import java.util.HashSet;

import net.sf.gluebooster.java.booster.basic.container.BoostedNode;
import net.sf.gluebooster.java.booster.basic.container.SimpleBoostedNode;

import org.junit.Assert;
import org.junit.Test;

public class RefactorTest {

	@Test
	public void testReplace() throws Exception {

		SimpleBoostedNode<String> node1 = Refactor
				.createSimpleBoostedNode("node 1");

		SimpleBoostedNode<String> node2 = Refactor
				.createSimpleBoostedNode("node 2");

		SimpleBoostedNode<String> node3 = Refactor
				.createSimpleBoostedNode("node 3");

		Refactor.addEdge(node1, node2);
		Refactor.addEdge(node2, node3);

		// System.out.println(Refactor.getStructureString(node1));
		// System.out.println(Refactor.getStructureString(node2));
		// System.out.println(Refactor.getStructureString(node3));

		// Remove the node 2 by replacing it with an empty graph
		Refactor.replace(node2, new BoostedNodeGraph());


		// System.out.println(Refactor.getStructureString(node1));
		// System.out.println(Refactor.getStructureString(node3));

		Refactor.checkPredecessorSuccessor(node1);
		Refactor.checkPredecessorSuccessor(node3);
		Assert.assertEquals(1, Refactor.getOutgoingEdgeCount(node1));
		Assert.assertEquals(1, Refactor.getIngoingEdgeCount(node3));

	}

	@Test
	public void testAddReachablePredecessors() throws Exception {

		SimpleBoostedNode<String> node1 = Refactor
				.createSimpleBoostedNode("node 1");
		SimpleBoostedNode<String> node2 = Refactor
				.createSimpleBoostedNode("node 2");
		SimpleBoostedNode<String> node3 = Refactor
				.createSimpleBoostedNode("node 3");
		SimpleBoostedNode<String> node4 = Refactor
				.createSimpleBoostedNode("node 4");

		Refactor.addEdge(node1, node2);
		Refactor.addEdge(node4, node2);
		Refactor.addEdge(node2, node3);

		HashSet<BoostedNode> predecessors = new HashSet<BoostedNode>();
		Refactor.addReachablePredecessors(node3, predecessors);

		Assert.assertTrue(predecessors.contains(node1));
		Assert.assertTrue(predecessors.contains(node2));
		Assert.assertTrue(predecessors.contains(node4));
	}
}
